@@ -15,14 +15,12 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335  USA
 */
 
+#include <my_global.h>
 #include <m_string.h>
 #include <mysql.h>
 #include <my_getopt.h>
 #include <my_dir.h>
-#include <my_global.h>
-#include <stdio.h>
-#include <string.h>
-
+#include <mysql_version.h>
 
 #define SHOW_VERSION "1.0.0"
 #define PRINT_VERSION do { printf("%s  Ver %s Distrib %s\n",    \
@@ -36,7 +34,7 @@ static uint opt_no_defaults= 0;
 static uint opt_print_defaults= 0;
 static char *opt_datadir=0, *opt_basedir=0,
             *opt_plugin_dir=0, *opt_plugin_ini=0,
-            *opt_mysqld=0, *opt_my_print_defaults=0;
+            *opt_mysqld=0, *opt_my_print_defaults=0, *opt_lc_messages_dir;
 static char bootstrap[FN_REFLEN];
 
 
@@ -72,6 +70,8 @@ static struct my_option my_long_options[] =
   {"my-print-defaults", 'f', "Path to my_print_defaults executable. "
    "Example: /source/temp11/extra",
     0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"lc-messages-dir", 'l', "The error messages dir for the server. ",
+    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"verbose", 'v',
     "More verbose output; you can use this multiple times to get even more "
     "verbose output.",
@@ -161,8 +161,7 @@ static int make_tempfile(char *filename, const char *ext)
 {
   int fd= 0;
 
-  if ((fd=create_temp_file(filename, NullS, ext, O_CREAT | O_WRONLY,
-         MYF(MY_WME))) < 0)
+  if ((fd= create_temp_file(filename, NullS, ext, 0, MYF(MY_WME))) < 0)
   {
     fprintf(stderr, "ERROR: Cannot generate temporary file. Error code: %d.\n",
             fd);
@@ -264,7 +263,7 @@ static char *convert_path(const char *argument)
   /* Convert / to \\ to make Windows paths */
   char *winfilename= my_strdup(argument, MYF(MY_FAE));
   char *pos, *end;
-  int length= strlen(argument);
+  size_t length= strlen(argument);
 
   for (pos= winfilename, end= pos+length ; pos < end ; pos++)
   {
@@ -282,7 +281,7 @@ static char *convert_path(const char *argument)
 
   @param[in]  path  The Windows path to examine.
 
-  @returns string containing excaped quotes if spaces found in path
+  @returns string containing escaped quotes if spaces found in path
 */
 static char *add_quotes(const char *path)
 {
@@ -308,6 +307,7 @@ static char *add_quotes(const char *path)
   --basedir
   --plugin-dir
   --plugin-ini
+  --lc-messages-dir
 
   These values are used if the user has not specified a value.
 
@@ -324,7 +324,7 @@ static int get_default_values()
   int ret= 0;
   FILE *file= 0;
 
-  bzero(tool_path, FN_REFLEN);
+  memset(tool_path, 0, FN_REFLEN);
   if ((error= find_tool("my_print_defaults" FN_EXEEXT, tool_path)))
     goto exit;
   else
@@ -337,9 +337,9 @@ static int get_default_values()
       char *format_str= 0;
   
       if (has_spaces(tool_path) || has_spaces(defaults_file))
-        format_str = "\"%s mysqld > %s\"";
+        format_str = "\"%s --mysqld > %s\"";
       else
-        format_str = "%s mysqld > %s";
+        format_str = "%s --mysqld > %s";
   
       snprintf(defaults_cmd, sizeof(defaults_cmd), format_str,
                add_quotes(tool_path), add_quotes(defaults_file));
@@ -350,7 +350,7 @@ static int get_default_values()
     }
 #else
     snprintf(defaults_cmd, sizeof(defaults_cmd),
-             "%s mysqld > %s", tool_path, defaults_file);
+             "%s --mysqld > %s", tool_path, defaults_file);
 #endif
 
     /* Execute the command */
@@ -367,6 +367,12 @@ static int get_default_values()
     }
     /* Now open the file and read the defaults we want. */
     file= fopen(defaults_file, "r");
+    if (file == NULL)
+    {
+      fprintf(stderr, "ERROR: failed to open file %s: %s.\n", defaults_file,
+              strerror(errno));
+      goto exit;
+    }
     while (fgets(line, FN_REFLEN, file) != NULL)
     {
       char *value= 0;
@@ -379,14 +385,20 @@ static int get_default_values()
       {
         opt_basedir= my_strdup(value, MYF(MY_FAE));
       }
-      if ((opt_plugin_dir == 0) && ((value= get_value(line, "--plugin_dir"))))
+      if ((opt_plugin_dir == 0) && ((value= get_value(line, "--plugin_dir")) ||
+          (value= get_value(line, "--plugin-dir"))))
       {
         opt_plugin_dir= my_strdup(value, MYF(MY_FAE));
       }
-      if ((opt_plugin_ini == 0) && ((value= get_value(line, "--plugin_ini"))))
+      if ((opt_lc_messages_dir == 0) &&
+          ((value= get_value(line, "--lc_messages_dir")) ||
+          (value= get_value(line, "--lc_messages-dir")) ||
+          (value= get_value(line, "--lc-messages_dir")) ||
+          (value= get_value(line, "--lc-messages-dir"))))
       {
-        opt_plugin_ini= my_strdup(value, MYF(MY_FAE));
+        opt_lc_messages_dir= my_strdup(value, MYF(MY_FAE));
       }
+
     }
   }
 exit:
@@ -426,6 +438,7 @@ static void usage(void)
   --basedir
   --plugin-dir
   --plugin-ini
+  --lc-messages-dir
 
 */
 
@@ -458,6 +471,10 @@ static void print_default_values(void)
   {
     printf("--my_print_defaults=%s ", opt_my_print_defaults);
   }
+  if (opt_lc_messages_dir)
+  {
+    printf("--lc_messages_dir=%s ", opt_lc_messages_dir);
+  }
   printf("\n");
 }
 
@@ -512,6 +529,10 @@ get_one_option(int optid,
   case 'f':
     opt_my_print_defaults= my_strdup(argument, MYF(MY_FAE));
     break;
+  case 'l':
+    opt_lc_messages_dir= my_strdup(argument, MYF(MY_FAE));
+    break;
+
   }
   return 0;
 }
@@ -714,11 +735,11 @@ static int check_options(int argc, char **argv, char *operation)
   
   /* Form prefix strings for the options. */
   const char *basedir_prefix = "--basedir=";
-  int basedir_len= strlen(basedir_prefix);
+  size_t basedir_len= strlen(basedir_prefix);
   const char *datadir_prefix = "--datadir=";
-  int datadir_len= strlen(datadir_prefix);
+  size_t datadir_len= strlen(datadir_prefix);
   const char *plugin_dir_prefix = "--plugin_dir=";
-  int plugin_dir_len= strlen(plugin_dir_prefix);
+  size_t plugin_dir_len= strlen(plugin_dir_prefix);
 
   strcpy(plugin_name, "");
   for (i = 0; i < argc && num_found < 5; i++)
@@ -897,6 +918,8 @@ static int process_options(int argc, char *argv[], char *operation)
     printf("# plugin_dir = %s\n", opt_plugin_dir);
     printf("#    datadir = %s\n", opt_datadir);
     printf("# plugin_ini = %s\n", opt_plugin_ini);
+    if (opt_lc_messages_dir != 0)
+      printf("# lc_messages_dir = %s\n", opt_lc_messages_dir);
   }
 
 exit:
@@ -954,6 +977,12 @@ static int check_access()
             opt_my_print_defaults);
     goto exit;
   }
+  if (opt_lc_messages_dir && (error= my_access(opt_lc_messages_dir, F_OK)))
+  {
+    fprintf(stderr, "ERROR: Cannot access lc-messages-dir path '%s'.\n",
+            opt_lc_messages_dir);
+    goto exit;
+  }
 
 exit:
   return error;
@@ -1005,7 +1034,7 @@ found:
 
 static int find_plugin(char *tp_path)
 {
-  /* Check for existance of plugin */
+  /* Check for existence of plugin */
   fn_format(tp_path, plugin_data.so_name, opt_plugin_dir, "", MYF(0));
   if (!file_exists(tp_path))
   {
@@ -1022,7 +1051,7 @@ static int find_plugin(char *tp_path)
 
 
 /**
-  Build the boostrap file.
+  Build the bootstrap file.
   
   Create a new file and populate it with SQL commands to ENABLE or DISABLE
   the plugin via REPLACE and DELETE operations on the mysql.plugin table.
@@ -1151,7 +1180,7 @@ exit:
   
   Create a command line sequence to launch mysqld in bootstrap mode. This
   will allow mysqld to launch a minimal server instance to read and
-  execute SQL commands from a file piped in (the boostrap file). We use
+  execute SQL commands from a file piped in (the bootstrap file). We use
   the --no-defaults option to skip reading values from the config file.
 
   The bootstrap mode skips loading of plugins and many other subsystems.
@@ -1183,18 +1212,33 @@ static int bootstrap_server(char *server_path, char *bootstrap_file)
     verbose_str= "";
   if (has_spaces(opt_datadir) || has_spaces(opt_basedir) ||
       has_spaces(bootstrap_file))
-    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s < %s\"";
-  else 
-    format_str= "%s %s --bootstrap --datadir=%s --basedir=%s < %s";
-
+  {
+    if (opt_lc_messages_dir != NULL)
+      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
+    else
+    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s <%s\"";
+  }
+  else
+  {
+    if (opt_lc_messages_dir != NULL)
+      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
+    else
+      format_str= "%s %s --bootstrap --datadir=%s --basedir=%s <%s";
+  }
   snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_str,
            add_quotes(convert_path(server_path)), verbose_str,
            add_quotes(opt_datadir), add_quotes(opt_basedir),
            add_quotes(bootstrap_file));
 #else
-  snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-           "%s --no-defaults --bootstrap --datadir=%s --basedir=%s"
-           " < %s", server_path, opt_datadir, opt_basedir, bootstrap_file);
+  if (opt_lc_messages_dir != NULL)
+    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
+             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s"
+             " <%s", server_path, opt_datadir, opt_basedir, opt_lc_messages_dir, bootstrap_file);
+  else
+    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
+             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s"
+             " <%s", server_path, opt_datadir, opt_basedir, bootstrap_file);
+
 #endif
 
   /* Execute the command */
