@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2001, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2017, MariaDB
+   Copyright (c) 2010, 2012, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
 */
 
 /* By Jani Tolonen, 2001-04-20, MySQL Development Team */
@@ -367,8 +367,12 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     print_version(); exit(0);
     break;
   case OPT_MYSQL_PROTOCOL:
-    opt_protocol= find_type_or_exit(argument, &sql_protocol_typelib,
-                                    opt->name);
+    if ((opt_protocol= find_type_with_warning(argument, &sql_protocol_typelib,
+                                              opt->name)) <= 0)
+    {
+      sf_leaking_memory= 1; /* no memory leak reports here */
+      exit(1);
+    }
     break;
   }
 
@@ -436,8 +440,10 @@ static int get_options(int *argc, char ***argv)
     else
       default_charset= (char*) MYSQL_AUTODETECT_CHARSET_NAME;
   }
-  if (strcmp(default_charset, MYSQL_AUTODETECT_CHARSET_NAME) &&
-      !get_charset_by_csname(default_charset, MY_CS_PRIMARY, MYF(MY_WME)))
+  if (!strcmp(default_charset, MYSQL_AUTODETECT_CHARSET_NAME))
+    default_charset= (char *)my_default_csname();
+
+  if (!get_charset_by_csname(default_charset, MY_CS_PRIMARY, MYF(MY_WME)))
   {
     printf("Unsupported character set: %s\n", default_charset);
     DBUG_RETURN(1);
@@ -537,7 +543,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
 {
   int view;
   char *table;
-  uint table_len;
+  size_t table_len;
   DBUG_ENTER("process_selected_tables");
 
   if (use_db(db))
@@ -581,6 +587,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
     my_free(table_names_comma_sep);
   }
   else
+  {
     for (; tables > 0; tables--, table_names++)
     {
       table= *table_names;
@@ -590,6 +597,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
         continue;
       handle_request_for_tables(table, table_len, view == 1, opt_all_in_1);
     }
+  }
   DBUG_RETURN(0);
 } /* process_selected_tables */
 
@@ -663,7 +671,7 @@ static int process_all_tables_in_db(char *database)
     size_t tot_length = 0;
 
     char *views, *views_end;
-    uint tot_views_length = 0;
+    size_t tot_views_length = 0;
 
     while ((row = mysql_fetch_row(res)))
     {
@@ -857,10 +865,11 @@ static int use_db(char *database)
   DBUG_RETURN(0);
 } /* use_db */
 
+/* Do not send commands to replication slaves. */
 static int disable_binlog()
 {
-  const char *stmt= "SET SQL_LOG_BIN=0";
-  return run_query(stmt, 0);
+  mysql_query(sock, "SET WSREP_ON=0"); /* ignore the error, if any */
+  return run_query("SET SQL_LOG_BIN=0", 0);
 }
 
 static int handle_request_for_tables(char *tables, size_t length,
@@ -908,16 +917,29 @@ static int handle_request_for_tables(char *tables, size_t length,
     }
     break;
   case DO_ANALYZE:
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run analyze on a view");
+      DBUG_RETURN(1);
+    }
     DBUG_ASSERT(!view);
     op= (opt_write_binlog) ? "ANALYZE" : "ANALYZE NO_WRITE_TO_BINLOG";
     if (opt_persistent_all) end = strmov(end, " PERSISTENT FOR ALL");
     break;
   case DO_OPTIMIZE:
-    DBUG_ASSERT(!view);
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run optimize on a view");
+      DBUG_RETURN(1);
+    }
     op= (opt_write_binlog) ? "OPTIMIZE" : "OPTIMIZE NO_WRITE_TO_BINLOG";
     break;
   case DO_FIX_NAMES:
-    DBUG_ASSERT(!view);
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run fix names on a view");
+      DBUG_RETURN(1);
+    }
     DBUG_RETURN(fix_table_storage_name(tables));
   }
 
@@ -946,9 +968,10 @@ static int handle_request_for_tables(char *tables, size_t length,
   }
   if (verbose >= 3)
     puts(query);
-  if (mysql_real_query(sock, query, query_length))
+  if (mysql_real_query(sock, query, (ulong)query_length))
   {
-    sprintf(message, "when executing '%s%s... %s'", op, tab_view, options);
+    my_snprintf(message, sizeof(message), "when executing '%s%s... %s'",
+                op, tab_view, options);
     DBerror(sock, message);
     my_free(query);
     DBUG_RETURN(1);
@@ -957,7 +980,7 @@ static int handle_request_for_tables(char *tables, size_t length,
   if (opt_flush_tables)
   {
     query_length= sprintf(query, "FLUSH TABLES %s", table_name);
-    if (mysql_real_query(sock, query, query_length))
+    if (mysql_real_query(sock, query, (ulong)query_length))
     {
       DBerror(sock, query);
       my_free(query);
@@ -983,7 +1006,6 @@ static void print_result()
   char prev[(NAME_LEN+9)*3+2];
   char prev_alter[MAX_ALTER_STR_SIZE];
   size_t length_of_db= strlen(sock->db);
-  uint i;
   my_bool found_error=0, table_rebuild=0;
   DYNAMIC_ARRAY *array4repair= &tables4repair;
   DBUG_ENTER("print_result");
@@ -992,7 +1014,7 @@ static void print_result()
 
   prev[0] = '\0';
   prev_alter[0]= 0;
-  for (i = 0; (row = mysql_fetch_row(res)); i++)
+  while ((row = mysql_fetch_row(res)))
   {
     int changed = strcmp(prev, row[0]);
     my_bool status = !strcmp(row[2], "status");
@@ -1072,6 +1094,7 @@ static void print_result()
 
 static int dbConnect(char *host, char *user, char *passwd)
 {
+  my_bool reconnect= 1;
   DBUG_ENTER("dbConnect");
   if (verbose > 1)
   {
@@ -1110,7 +1133,7 @@ static int dbConnect(char *host, char *user, char *passwd)
     DBerror(&mysql_connection, "when trying to connect");
     DBUG_RETURN(1);
   }
-  mysql_connection.reconnect= 1;
+  mysql_options(&mysql_connection, MYSQL_OPT_RECONNECT, &reconnect);
   DBUG_RETURN(0);
 } /* dbConnect */
 
@@ -1160,9 +1183,7 @@ int main(int argc, char **argv)
   /*
   ** Check out the args
   */
-  if (load_defaults("my", load_default_groups, &argc, &argv))
-    goto end2;
-
+  load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   defaults_argv= argv;
   if (get_options(&argc, &argv))
     goto end1;
@@ -1238,7 +1259,6 @@ int main(int argc, char **argv)
   my_free(shared_memory_base_name);
   mysql_library_end();
   free_defaults(defaults_argv);
- end2:
   my_end(my_end_arg);
   return ret;
 } /* main */
