@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2015, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2016, MariaDB
+   Copyright (c) 2010, 2017, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
 */
 
 /* Show databases, tables or columns */
@@ -68,11 +68,11 @@ int main(int argc, char **argv)
   my_bool first_argument_uses_wildcards=0;
   char *wild;
   MYSQL mysql;
+  my_bool reconnect;
   static char **defaults_argv;
   MY_INIT(argv[0]);
   sf_leaking_memory=1; /* don't report memory leaks on early exits */
-  if (load_defaults("my",load_default_groups,&argc,&argv))
-    exit(1);
+  load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   defaults_argv=argv;
 
   get_options(&argc,&argv);
@@ -135,6 +135,8 @@ int main(int argc, char **argv)
   if (shared_memory_base_name)
     mysql_options(&mysql,MYSQL_SHARED_MEMORY_BASE_NAME,shared_memory_base_name);
 #endif
+  if (!strcmp(default_charset,MYSQL_AUTODETECT_CHARSET_NAME))
+    default_charset= (char *)my_default_csname();
   mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, default_charset);
 
   if (opt_plugin_dir && *opt_plugin_dir)
@@ -155,7 +157,8 @@ int main(int argc, char **argv)
     error= 1;
     goto error;
   }
-  mysql.reconnect= 1;
+  reconnect= 1;
+  mysql_options(&mysql, MYSQL_OPT_RECONNECT, &reconnect);
 
   switch (argc) {
   case 0:  error=list_dbs(&mysql,wild); break;
@@ -328,8 +331,12 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 #endif
     break;
   case OPT_MYSQL_PROTOCOL:
-    opt_protocol= find_type_or_exit(argument, &sql_protocol_typelib,
-                                    opt->name);
+    if ((opt_protocol= find_type_with_warning(argument, &sql_protocol_typelib,
+                                              opt->name)) <= 0)
+    {
+      sf_leaking_memory= 1; /* no memory leak reports here */
+      exit(1);
+    }
     break;
   case '#':
     DBUG_PUSH(argument ? argument : "d:t:o");
@@ -655,7 +662,7 @@ static int
 list_table_status(MYSQL *mysql,const char *db,const char *wild)
 {
   char query[NAME_LEN + 100];
-  int len;
+  size_t len;
   MYSQL_RES *result;
   MYSQL_ROW row;
 
@@ -902,7 +909,7 @@ static void print_res_header(MYSQL_RES *result)
 
 static void print_res_top(MYSQL_RES *result)
 {
-  uint i,length;
+  size_t i,length;
   MYSQL_FIELD *field;
 
   putchar('+');
@@ -910,7 +917,7 @@ static void print_res_top(MYSQL_RES *result)
   while((field = mysql_fetch_field(result)))
   {
     if ((length= strlen(field->name)) > field->max_length)
-      field->max_length=length;
+      field->max_length=(ulong)length;
     else
       length=field->max_length;
     for (i=length+2 ; i--> 0 ; )
