@@ -147,10 +147,34 @@ static inline void bitmap_unlock(MY_BITMAP *map __attribute__((unused)))
 }
 
 
-my_bool bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
-		    my_bool thread_safe __attribute__((unused)))
+static inline uint get_first_set(my_bitmap_map value, uint word_pos)
 {
-  DBUG_ENTER("bitmap_init");
+  uchar *byte_ptr= (uchar*)&value;
+  uchar byte_value;
+  uint byte_pos, bit_pos;
+
+  DBUG_ASSERT(value);
+  for (byte_pos=0; ; byte_pos++, byte_ptr++)
+  {
+    if ((byte_value= *byte_ptr))
+    {
+      for (bit_pos=0; ; bit_pos++)
+        if (byte_value & (1 << bit_pos))
+          return (word_pos*32) + (byte_pos*8) + bit_pos;
+    }
+  }
+  return MY_BIT_NONE;                           /* Impossible */
+}
+
+/*
+  Initialize a bitmap object. All bits will be set to zero
+*/
+
+my_bool my_bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
+                       my_bool thread_safe)
+{
+  DBUG_ENTER("my_bitmap_init");
+  map->mutex= 0;
   if (!buf)
   {
     uint size_in_bytes= bitmap_buffer_size(n_bits);
@@ -160,7 +184,6 @@ my_bool bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
       size_in_bytes= ALIGN_SIZE(size_in_bytes);
       extra= sizeof(mysql_mutex_t);
     }
-    map->mutex= 0;
     if (!(buf= (my_bitmap_map*) my_malloc(size_in_bytes+extra, MYF(MY_WME))))
       DBUG_RETURN(1);
     if (thread_safe)
@@ -182,9 +205,9 @@ my_bool bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
 }
 
 
-void bitmap_free(MY_BITMAP *map)
+void my_bitmap_free(MY_BITMAP *map)
 {
-  DBUG_ENTER("bitmap_free");
+  DBUG_ENTER("my_bitmap_free");
   if (map->bitmap)
   {
     if (map->mutex)
@@ -235,7 +258,8 @@ my_bool bitmap_fast_test_and_set(MY_BITMAP *map, uint bitmap_bit)
 my_bool bitmap_test_and_set(MY_BITMAP *map, uint bitmap_bit)
 {
   my_bool res;
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(bitmap_bit < map->n_bits);
   bitmap_lock(map);
   res= bitmap_fast_test_and_set(map, bitmap_bit);
   bitmap_unlock(map);
@@ -268,7 +292,8 @@ my_bool bitmap_fast_test_and_clear(MY_BITMAP *map, uint bitmap_bit)
 my_bool bitmap_test_and_clear(MY_BITMAP *map, uint bitmap_bit)
 {
   my_bool res;
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(bitmap_bit < map->n_bits);
   bitmap_lock(map);
   res= bitmap_fast_test_and_clear(map, bitmap_bit);
   bitmap_unlock(map);
@@ -286,24 +311,31 @@ uint bitmap_set_next(MY_BITMAP *map)
 }
 
 
+/**
+  Set the specified number of bits in the bitmap buffer.
+
+  @param map         [IN]       Bitmap
+  @param prefix_size [IN]       Number of bits to be set
+*/
 void bitmap_set_prefix(MY_BITMAP *map, uint prefix_size)
 {
   uint prefix_bytes, prefix_bits, d;
   uchar *m= (uchar *)map->bitmap;
 
-  DBUG_ASSERT(map->bitmap &&
-	      (prefix_size <= map->n_bits || prefix_size == (uint) ~0));
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(prefix_size <= map->n_bits || prefix_size == (uint) ~0);
   set_if_smaller(prefix_size, map->n_bits);
   if ((prefix_bytes= prefix_size / 8))
     memset(m, 0xff, prefix_bytes);
   m+= prefix_bytes;
   if ((prefix_bits= prefix_size & 7))
   {
-    *m++= (1 << prefix_bits)-1;
-    prefix_bytes++;
+    *(m++)= (1 << prefix_bits)-1;
+    // As the prefix bits are set, lets count this byte too as a prefix byte.
+    prefix_bytes ++;
   }
   if ((d= no_bytes_in_map(map)-prefix_bytes))
-    bzero(m, d);
+    memset(m, 0, d);
 }
 
 
@@ -313,7 +345,8 @@ my_bool bitmap_is_prefix(const MY_BITMAP *map, uint prefix_size)
   uchar *m= (uchar*) map->bitmap;
   uchar *end_prefix= m+(prefix_size-1)/8;
   uchar *end;
-  DBUG_ASSERT(m && prefix_size <= map->n_bits);
+  DBUG_ASSERT(m);
+  DBUG_ASSERT(prefix_size <= map->n_bits);
 
   /* Empty prefix is always true */
   if (!prefix_size)
@@ -353,6 +386,7 @@ my_bool bitmap_is_clear_all(const MY_BITMAP *map)
   my_bitmap_map *data_ptr= map->bitmap;
   my_bitmap_map *end= map->last_word_ptr;
 
+  DBUG_ASSERT(map->n_bits > 0);
   for (; data_ptr < end; data_ptr++)
     if (*data_ptr)
       return FALSE;
@@ -365,8 +399,8 @@ my_bool bitmap_is_subset(const MY_BITMAP *map1, const MY_BITMAP *map2)
 {
   my_bitmap_map *m1= map1->bitmap, *m2= map2->bitmap, *end;
 
-  DBUG_ASSERT(map1->bitmap && map2->bitmap &&
-              map1->n_bits==map2->n_bits);
+  DBUG_ASSERT(map1->bitmap && map2->bitmap);
+  DBUG_ASSERT(map1->n_bits==map2->n_bits);
 
   end= map1->last_word_ptr;
   while (m1 < end)
@@ -384,8 +418,9 @@ my_bool bitmap_is_overlapping(const MY_BITMAP *map1, const MY_BITMAP *map2)
 {
   my_bitmap_map *m1= map1->bitmap, *m2= map2->bitmap, *end;
 
-  DBUG_ASSERT(map1->bitmap && map2->bitmap &&
-              map1->n_bits==map2->n_bits);
+  DBUG_ASSERT(map1->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map1->n_bits==map2->n_bits);
 
   end= map1->last_word_ptr;
   while (m1 < end)
@@ -403,9 +438,10 @@ void bitmap_intersect(MY_BITMAP *map, const MY_BITMAP *map2)
   my_bitmap_map *to= map->bitmap, *from= map2->bitmap, *end;
   uint len= no_words_in_map(map), len2 = no_words_in_map(map2);
 
-  DBUG_ASSERT(map->bitmap && map2->bitmap);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(map2->bitmap);
 
-  end= to+min(len,len2);
+  end= to+MY_MIN(len,len2);
   while (to < end)
     *to++ &= *from++;
 
@@ -448,7 +484,8 @@ my_bool bitmap_exists_intersection(const MY_BITMAP **bitmap_array,
   uint i, j, start_idx, end_idx;
   my_bitmap_map cur_res;
 
-  DBUG_ASSERT(bitmap_count && end_bit >= start_bit);
+  DBUG_ASSERT(bitmap_count);
+  DBUG_ASSERT(end_bit >= start_bit);
   for (j= 0; j < bitmap_count; j++)
     DBUG_ASSERT(end_bit < bitmap_array[j]->n_bits);
 
@@ -476,8 +513,9 @@ my_bool bitmap_union_is_set_all(const MY_BITMAP *map1, const MY_BITMAP *map2)
 {
   my_bitmap_map *m1= map1->bitmap, *m2= map2->bitmap, *end;
 
-  DBUG_ASSERT(map1->bitmap && map2->bitmap &&
-              map1->n_bits==map2->n_bits);
+  DBUG_ASSERT(map1->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map1->n_bits==map2->n_bits);
   end= map1->last_word_ptr;
   while ( m1 < end)
     if ((*m1++ | *m2++) != 0xFFFFFFFF)
@@ -522,8 +560,9 @@ void bitmap_set_above(MY_BITMAP *map, uint from_byte, uint use_bit)
 void bitmap_subtract(MY_BITMAP *map, const MY_BITMAP *map2)
 {
   my_bitmap_map *to= map->bitmap, *from= map2->bitmap, *end;
-  DBUG_ASSERT(map->bitmap && map2->bitmap &&
-              map->n_bits==map2->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map->n_bits==map2->n_bits);
 
   end= map->last_word_ptr;
 
@@ -536,8 +575,9 @@ void bitmap_union(MY_BITMAP *map, const MY_BITMAP *map2)
 {
   my_bitmap_map *to= map->bitmap, *from= map2->bitmap, *end;
 
-  DBUG_ASSERT(map->bitmap && map2->bitmap &&
-              map->n_bits==map2->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map->n_bits == map2->n_bits);
   end= map->last_word_ptr;
 
   while (to <= end)
@@ -548,8 +588,9 @@ void bitmap_union(MY_BITMAP *map, const MY_BITMAP *map2)
 void bitmap_xor(MY_BITMAP *map, const MY_BITMAP *map2)
 {
   my_bitmap_map *to= map->bitmap, *from= map2->bitmap, *end= map->last_word_ptr;
-  DBUG_ASSERT(map->bitmap && map2->bitmap &&
-              map->n_bits==map2->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map->n_bits == map2->n_bits);
   while (to <= end)
     *to++ ^= *from++;
 }
@@ -586,8 +627,9 @@ void bitmap_copy(MY_BITMAP *map, const MY_BITMAP *map2)
 {
   my_bitmap_map *to= map->bitmap, *from= map2->bitmap, *end;
 
-  DBUG_ASSERT(map->bitmap && map2->bitmap &&
-              map->n_bits==map2->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(map2->bitmap);
+  DBUG_ASSERT(map->n_bits == map2->n_bits);
   end= map->last_word_ptr;
 
   while (to <= end)
@@ -597,12 +639,10 @@ void bitmap_copy(MY_BITMAP *map, const MY_BITMAP *map2)
 
 uint bitmap_get_first_set(const MY_BITMAP *map)
 {
-  uchar *byte_ptr;
-  uint i,j,k;
-  my_bitmap_map *data_ptr, *end= map->last_word_ptr;
+  uint i;
+  my_bitmap_map *data_ptr= map->bitmap, *end= map->last_word_ptr;
 
   DBUG_ASSERT(map->bitmap);
-  data_ptr= map->bitmap;
 
   for (i=0; data_ptr < end; data_ptr++, i++)
     if (*data_ptr)
@@ -611,25 +651,66 @@ uint bitmap_get_first_set(const MY_BITMAP *map)
     return MY_BIT_NONE;
 
 found:
-  {
-    byte_ptr= (uchar*)data_ptr;
-    for (j=0; ; j++, byte_ptr++)
-    {
-      if (*byte_ptr)
-      {
-        for (k=0; ; k++)
-        {
-          if (*byte_ptr & (1 << k))
-            return (i*32) + (j*8) + k;
-        }
-      }
-    }
-  }
-  DBUG_ASSERT(0);
-  return MY_BIT_NONE;                           /* Impossible */
+  return get_first_set(*data_ptr, i);
 }
 
 
+/**
+  Get the next set bit.
+
+  @param  map         Bitmap
+  @param  bitmap_bit  Bit to start search from
+
+  @return Index to first bit set after bitmap_bit
+*/
+
+uint bitmap_get_next_set(const MY_BITMAP *map, uint bitmap_bit)
+{
+  uint word_pos, byte_to_mask, i;
+  union { my_bitmap_map bitmap ; uchar bitmap_buff[sizeof(my_bitmap_map)]; }
+  first_word;
+  uchar *ptr= &first_word.bitmap_buff[0];
+  my_bitmap_map *data_ptr, *end= map->last_word_ptr;
+
+  DBUG_ASSERT(map->bitmap);
+
+  /* Look for the next bit */
+  bitmap_bit++;
+  if (bitmap_bit >= map->n_bits)
+    return MY_BIT_NONE;
+  word_pos= bitmap_bit / 32;
+  data_ptr= map->bitmap + word_pos;
+  first_word.bitmap= *data_ptr;
+
+  /* Mask out previous bits from first_word */
+  byte_to_mask= (bitmap_bit % 32) / 8;
+  for (i= 0; i < byte_to_mask; i++)
+    ptr[i]= 0;
+  ptr[byte_to_mask]&= 0xFFU << (bitmap_bit & 7);
+
+  if (data_ptr == end)
+  {
+    if (first_word.bitmap & ~map->last_word_mask)
+      return get_first_set(first_word.bitmap, word_pos);
+    else
+      return MY_BIT_NONE;
+  }
+   
+  if (first_word.bitmap)
+    return get_first_set(first_word.bitmap, word_pos);
+
+  for (data_ptr++, word_pos++; data_ptr < end; data_ptr++, word_pos++)
+    if (*data_ptr)
+      return get_first_set(*data_ptr, word_pos);
+
+  if (!(*end & ~map->last_word_mask))
+    return MY_BIT_NONE;
+  return get_first_set(*end, word_pos);
+}
+
+
+/* Get first free bit */
+
 uint bitmap_get_first(const MY_BITMAP *map)
 {
   uchar *byte_ptr;
@@ -647,17 +728,15 @@ uint bitmap_get_first(const MY_BITMAP *map)
     return MY_BIT_NONE;
 
 found:
+  byte_ptr= (uchar*)data_ptr;
+  for (j=0; ; j++, byte_ptr++)
   {
-    byte_ptr= (uchar*)data_ptr;
-    for (j=0; ; j++, byte_ptr++)
+    if (*byte_ptr != 0xFF)
     {
-      if (*byte_ptr != 0xFF)
+      for (k=0; ; k++)
       {
-        for (k=0; ; k++)
-        {
-          if (!(*byte_ptr & (1 << k)))
-            return (i*32) + (j*8) + k;
-        }
+        if (!(*byte_ptr & (1 << k)))
+          return (i*32) + (j*8) + k;
       }
     }
   }
@@ -679,7 +758,8 @@ uint bitmap_lock_set_next(MY_BITMAP *map)
 void bitmap_lock_clear_bit(MY_BITMAP *map, uint bitmap_bit)
 {
   bitmap_lock(map);
-  DBUG_ASSERT(map->bitmap && bitmap_bit < map->n_bits);
+  DBUG_ASSERT(map->bitmap);
+  DBUG_ASSERT(bitmap_bit < map->n_bits);
   bitmap_clear_bit(map, bitmap_bit);
   bitmap_unlock(map);
 }
