@@ -11,13 +11,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1335  USA */
 
 #ifndef INJECTOR_H
 #define INJECTOR_H
 
 /* Pull in 'byte', 'my_off_t', and 'uint32' */
-#include <my_global.h>
 #include <my_bitmap.h>
 
 #include "rpl_constants.h"
@@ -117,8 +116,8 @@ public:
       class table 
       {
       public:
-        table(TABLE *table, bool is_transactional) 
-            : m_table(table), m_is_transactional(is_transactional)
+        table(TABLE *table, bool is_transactional_arg)
+            : m_table(table), m_is_transactional(is_transactional_arg)
         { 
         }
 
@@ -180,27 +179,6 @@ public:
        */
       int use_table(server_id_type sid, table tbl);
 
-      /*
-        Add a 'write row' entry to the transaction.
-      */
-      int write_row (server_id_type sid, table tbl, 
-                     MY_BITMAP const *cols, size_t colcnt,
-                     record_type record);
-
-      /*
-        Add a 'delete row' entry to the transaction.
-      */
-      int delete_row(server_id_type sid, table tbl, 
-                     MY_BITMAP const *cols, size_t colcnt,
-                     record_type record);
-
-      /*
-        Add an 'update row' entry to the transaction.
-      */
-      int update_row(server_id_type sid, table tbl, 
-                     MY_BITMAP const *cols, size_t colcnt,
-                     record_type before, record_type after);
-
       /*
         Commit a transaction.
 
@@ -324,7 +302,7 @@ public:
     void        new_trans(THD *, transaction *);
 
     int record_incident(THD*, Incident incident);
-    int record_incident(THD*, Incident incident, LEX_STRING const message);
+    int record_incident(THD*, Incident incident, const LEX_CSTRING *message);
 
 private:
     explicit injector();
