@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2010, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2013 Monty Program Ab.
+   Copyright (c) 2009, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,11 +17,10 @@
 
 /* Functions to handle date and time */
 
-#include "sql_priv.h"
-#include "unireg.h"                      // REQUIRED by other includes
+#include "mariadb.h"
 #include "sql_time.h"
-#include "tztime.h"                             // struct Time_zone
-#include "sql_class.h"   // THD, MODE_INVALID_DATES, MODE_NO_ZERO_DATE
+#include "tztime.h"                      // struct Time_zone
+#include "sql_class.h"                   // THD
 #include <m_ctype.h>
 
 
@@ -37,34 +36,108 @@
   Order of elements in 'interval_type_to_name' should correspond to 
   the order of elements in 'interval_type' enum
   
-  See also interval_type, interval_names
+  See also interval_type, interval_names, append_interval
 */
 
-LEX_STRING interval_type_to_name[INTERVAL_LAST] = {
-  { C_STRING_WITH_LEN("YEAR")},
-  { C_STRING_WITH_LEN("QUARTER")},
-  { C_STRING_WITH_LEN("MONTH")},
-  { C_STRING_WITH_LEN("WEEK")},
-  { C_STRING_WITH_LEN("DAY")},
-  { C_STRING_WITH_LEN("HOUR")},
-  { C_STRING_WITH_LEN("MINUTE")},
-  { C_STRING_WITH_LEN("SECOND")},
-  { C_STRING_WITH_LEN("MICROSECOND")},
-  { C_STRING_WITH_LEN("YEAR_MONTH")},
-  { C_STRING_WITH_LEN("DAY_HOUR")},
-  { C_STRING_WITH_LEN("DAY_MINUTE")},
-  { C_STRING_WITH_LEN("DAY_SECOND")},
-  { C_STRING_WITH_LEN("HOUR_MINUTE")},
-  { C_STRING_WITH_LEN("HOUR_SECOND")},
-  { C_STRING_WITH_LEN("MINUTE_SECOND")},
-  { C_STRING_WITH_LEN("DAY_MICROSECOND")},
-  { C_STRING_WITH_LEN("HOUR_MICROSECOND")},
-  { C_STRING_WITH_LEN("MINUTE_MICROSECOND")},
-  { C_STRING_WITH_LEN("SECOND_MICROSECOND")}
-}; 
+LEX_CSTRING interval_type_to_name[INTERVAL_LAST] = {
+  { STRING_WITH_LEN("YEAR")},
+  { STRING_WITH_LEN("QUARTER")},
+  { STRING_WITH_LEN("MONTH")},
+  { STRING_WITH_LEN("WEEK")},
+  { STRING_WITH_LEN("DAY")},
+  { STRING_WITH_LEN("HOUR")},
+  { STRING_WITH_LEN("MINUTE")},
+  { STRING_WITH_LEN("SECOND")},
+  { STRING_WITH_LEN("MICROSECOND")},
+  { STRING_WITH_LEN("YEAR_MONTH")},
+  { STRING_WITH_LEN("DAY_HOUR")},
+  { STRING_WITH_LEN("DAY_MINUTE")},
+  { STRING_WITH_LEN("DAY_SECOND")},
+  { STRING_WITH_LEN("HOUR_MINUTE")},
+  { STRING_WITH_LEN("HOUR_SECOND")},
+  { STRING_WITH_LEN("MINUTE_SECOND")},
+  { STRING_WITH_LEN("DAY_MICROSECOND")},
+  { STRING_WITH_LEN("HOUR_MICROSECOND")},
+  { STRING_WITH_LEN("MINUTE_MICROSECOND")},
+  { STRING_WITH_LEN("SECOND_MICROSECOND")}
+};
 
-	/* Calc weekday from daynr */
-	/* Returns 0 for monday, 1 for tuesday .... */
+int append_interval(String *str, interval_type int_type, const INTERVAL &interval)
+{
+  char buf[64];
+  size_t len;
+  switch (int_type) {
+  case INTERVAL_YEAR:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.year);
+    break;
+  case INTERVAL_QUARTER:
+  case INTERVAL_MONTH:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.month);
+    int_type=INTERVAL_MONTH;
+    break;
+  case INTERVAL_WEEK:
+  case INTERVAL_DAY:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.day);
+    int_type=INTERVAL_DAY;
+    break;
+  case INTERVAL_HOUR:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.hour);
+    break;
+  case INTERVAL_MINUTE:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.minute);
+    break;
+  case INTERVAL_SECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.second);
+    break;
+  case INTERVAL_MICROSECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.second_part);
+    break;
+  case INTERVAL_YEAR_MONTH:
+    len= my_snprintf(buf,sizeof(buf),"%u-%02u", interval.day, interval.month);
+    break;
+  case INTERVAL_DAY_HOUR:
+    len= my_snprintf(buf,sizeof(buf),"%u %u", interval.day, interval.hour);
+    break;
+  case INTERVAL_DAY_MINUTE:
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u", interval.day, interval.hour, interval.minute);
+    break;
+  case INTERVAL_DAY_SECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u:%02u", interval.day, interval.hour, interval.minute, interval.second);
+    break;
+  case INTERVAL_HOUR_MINUTE:
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u", interval.hour, interval.minute);
+    break;
+  case INTERVAL_HOUR_SECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u:%02u", interval.hour, interval.minute, interval.second);
+    break;
+  case INTERVAL_MINUTE_SECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u", interval.minute, interval.second);
+    break;
+  case INTERVAL_DAY_MICROSECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u:%02u.%06u", interval.day, interval.hour, interval.minute, interval.second, interval.second_part);
+    break;
+  case INTERVAL_HOUR_MICROSECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u:%02u.%06u", interval.hour, interval.minute, interval.second, interval.second_part);
+    break;
+  case INTERVAL_MINUTE_MICROSECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u.%06u", interval.minute, interval.second, interval.second_part);
+    break;
+  case INTERVAL_SECOND_MICROSECOND:
+    len= my_snprintf(buf,sizeof(buf),"%u.%06u", interval.second, interval.second_part);
+    break;
+  default:
+    DBUG_ASSERT(0);
+    len= 0;
+  }
+  return str->append(buf, len) || str->append(' ') ||
+         str->append(interval_type_to_name + int_type);
+}
+
+
+/*
+  Calc weekday from daynr
+  Returns 0 for monday, 1 for tuesday ...
+*/
 
 int calc_weekday(long daynr,bool sunday_first_day_of_week)
 {
@@ -106,9 +179,9 @@ uint calc_week(MYSQL_TIME *l_time, uint week_behaviour, uint *year)
   uint days;
   ulong daynr=calc_daynr(l_time->year,l_time->month,l_time->day);
   ulong first_daynr=calc_daynr(l_time->year,1,1);
-  bool monday_first= test(week_behaviour & WEEK_MONDAY_FIRST);
-  bool week_year= test(week_behaviour & WEEK_YEAR);
-  bool first_weekday= test(week_behaviour & WEEK_FIRST_WEEKDAY);
+  bool monday_first= MY_TEST(week_behaviour & WEEK_MONDAY_FIRST);
+  bool week_year= MY_TEST(week_behaviour & WEEK_YEAR);
+  bool first_weekday= MY_TEST(week_behaviour & WEEK_FIRST_WEEKDAY);
 
   uint weekday=calc_weekday(first_daynr, !monday_first);
   *year=l_time->year;
@@ -218,12 +291,12 @@ bool
 check_date_with_warn(const MYSQL_TIME *ltime, ulonglong fuzzy_date,
                      timestamp_type ts_type)
 {
-  int dummy_warnings;
-  if (check_date(ltime, fuzzy_date, &dummy_warnings))
+  int unused;
+  if (check_date(ltime, fuzzy_date, &unused))
   {
     ErrConvTime str(ltime);
-    make_truncated_value_warning(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                                 &str, ts_type, 0);
+    make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
+                                 &str, ts_type, NULL, NULL, NULL);
     return true;
   }
   return false;
@@ -239,8 +312,8 @@ adjust_time_range_with_warn(MYSQL_TIME *ltime, uint dec)
   if (check_time_range(ltime, dec, &warnings))
     return true;
   if (warnings)
-    make_truncated_value_warning(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                                 &str, MYSQL_TIMESTAMP_TIME, NullS);
+    make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
+                                 &str, MYSQL_TIMESTAMP_TIME, NULL, NULL, NULL);
   return false;
 }
 
@@ -256,8 +329,8 @@ adjust_time_range_with_warn(MYSQL_TIME *ltime, uint dec)
 */
 static uint
 to_ascii(CHARSET_INFO *cs,
-         const char *src, uint src_length,
-         char *dst, uint dst_length)
+         const char *src, size_t src_length,
+         char *dst, size_t dst_length)
                      
 {
   int cnvres;
@@ -274,14 +347,14 @@ to_ascii(CHARSET_INFO *cs,
     *dst++= static_cast<char>(wc);
   }
   *dst= '\0';
-  return dst - dst0;
+  return (uint)(dst - dst0);
 }
 
 
 /* Character set-aware version of str_to_time() */
-timestamp_type
-str_to_time(CHARSET_INFO *cs, const char *str,uint length,
-                 MYSQL_TIME *l_time, ulonglong fuzzydate, int *warning)
+bool
+str_to_time(CHARSET_INFO *cs, const char *str, size_t length,
+            MYSQL_TIME *l_time, ulonglong fuzzydate, MYSQL_TIME_STATUS *status)
 {
   char cnv[32];
   if ((cs->state & MY_CS_NONASCII) != 0)
@@ -289,14 +362,14 @@ str_to_time(CHARSET_INFO *cs, const char *str,uint length,
     length= to_ascii(cs, str, length, cnv, sizeof(cnv));
     str= cnv;
   }
-  return str_to_time(str, length, l_time, fuzzydate, warning);
+  return str_to_time(str, length, l_time, fuzzydate, status);
 }
 
 
 /* Character set-aware version of str_to_datetime() */
-timestamp_type str_to_datetime(CHARSET_INFO *cs,
-                               const char *str, uint length,
-                               MYSQL_TIME *l_time, ulonglong flags, int *was_cut)
+bool str_to_datetime(CHARSET_INFO *cs, const char *str, size_t length,
+                     MYSQL_TIME *l_time, ulonglong flags,
+                     MYSQL_TIME_STATUS *status)
 {
   char cnv[32];
   if ((cs->state & MY_CS_NONASCII) != 0)
@@ -304,7 +377,7 @@ timestamp_type str_to_datetime(CHARSET_INFO *cs,
     length= to_ascii(cs, str, length, cnv, sizeof(cnv));
     str= cnv;
   }
-  return str_to_datetime(str, length, l_time, flags, was_cut);
+  return str_to_datetime(str, length, l_time, flags, status);
 }
 
 
@@ -316,26 +389,25 @@ timestamp_type str_to_datetime(CHARSET_INFO *cs,
     See description of str_to_datetime() for more information.
 */
 
-timestamp_type
+bool
 str_to_datetime_with_warn(CHARSET_INFO *cs,
-                          const char *str, uint length, MYSQL_TIME *l_time,
+                          const char *str, size_t length, MYSQL_TIME *l_time,
                           ulonglong flags)
 {
-  int was_cut;
+  MYSQL_TIME_STATUS status;
   THD *thd= current_thd;
-  timestamp_type ts_type;
-  
-  ts_type= str_to_datetime(cs, str, length, l_time,
-                           (flags | (sql_mode_for_dates(thd))),
-                           &was_cut);
-  if (was_cut || ts_type <= MYSQL_TIMESTAMP_ERROR)
-    make_truncated_value_warning(thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+  bool ret_val= str_to_datetime(cs, str, length, l_time, flags, &status);
+  if (ret_val || status.warnings)
+    make_truncated_value_warning(thd,
+                                 ret_val ? Sql_condition::WARN_LEVEL_WARN :
+                                 Sql_condition::time_warn_level(status.warnings),
                                  str, length, flags & TIME_TIME_ONLY ?
-                                 MYSQL_TIMESTAMP_TIME : ts_type, NullS);
+                                 MYSQL_TIMESTAMP_TIME : l_time->time_type, 
+                                 NULL, NULL, NULL);
   DBUG_EXECUTE_IF("str_to_datetime_warn",
-                  push_warning(thd, MYSQL_ERROR::WARN_LEVEL_NOTE,
+                  push_warning(thd, Sql_condition::WARN_LEVEL_NOTE,
                                ER_YES, str););
-  return ts_type;
+  return ret_val;
 }
 
 
@@ -355,23 +427,23 @@ str_to_datetime_with_warn(CHARSET_INFO *cs,
 static bool number_to_time_with_warn(bool neg, ulonglong nr, ulong sec_part,
                                      MYSQL_TIME *ltime, ulonglong fuzzydate,
                                      const ErrConv *str,
-                                     const char *field_name)
+                                     const TABLE_SHARE *s, const char *field_name)
 {
   int was_cut;
   longlong res;
-  enum_field_types f_type;
-  bool have_warnings;
+  enum_mysql_timestamp_type ts_type;
+  bool have_warnings= false;
 
   if (fuzzydate & TIME_TIME_ONLY)
   {
     fuzzydate= TIME_TIME_ONLY; // clear other flags
-    f_type= MYSQL_TYPE_TIME;
+    ts_type= MYSQL_TIMESTAMP_TIME;
     res= number_to_time(neg, nr, sec_part, ltime, &was_cut);
     have_warnings= MYSQL_TIME_WARN_HAVE_WARNINGS(was_cut);
   }
   else
   {
-    f_type= MYSQL_TYPE_DATETIME;
+    ts_type= MYSQL_TIMESTAMP_DATETIME;
     if (neg)
     {
       res= -1;
@@ -386,17 +458,18 @@ static bool number_to_time_with_warn(bool neg, ulonglong nr, ulong sec_part,
   if (res < 0 || have_warnings)
   {
     make_truncated_value_warning(current_thd,
-                                 MYSQL_ERROR::WARN_LEVEL_WARN, str,
-                                 res < 0 ? MYSQL_TIMESTAMP_ERROR
-                                         : mysql_type_to_time_type(f_type),
-                                 field_name);
+                                 Sql_condition::WARN_LEVEL_WARN, str,
+                                 res < 0 ? MYSQL_TIMESTAMP_ERROR : ts_type,
+                                 s ? s->db.str : NULL,
+                                 s ? s->table_name.str : NULL, field_name);
   }
   return res < 0;
 }
 
 
 bool double_to_datetime_with_warn(double value, MYSQL_TIME *ltime,
-                                  ulonglong fuzzydate, const char *field_name)
+                                  ulonglong fuzzydate,
+                                  const TABLE_SHARE *s, const char *field_name)
 {
   const ErrConvDouble str(value);
   bool neg= value < 0;
@@ -404,34 +477,36 @@ bool double_to_datetime_with_warn(double value, MYSQL_TIME *ltime,
   if (neg)
     value= -value;
 
-  if (value > LONGLONG_MAX)
+  if (value > static_cast<double>(LONGLONG_MAX))
     value= static_cast<double>(LONGLONG_MAX);
 
   longlong nr= static_cast<ulonglong>(floor(value));
   uint sec_part= static_cast<ulong>((value - floor(value))*TIME_SECOND_PART_FACTOR);
   return number_to_time_with_warn(neg, nr, sec_part, ltime, fuzzydate, &str,
-                                  field_name);
+                                  s, field_name);
 }
 
 
 bool decimal_to_datetime_with_warn(const my_decimal *value, MYSQL_TIME *ltime,
-                                   ulonglong fuzzydate, const char *field_name)
+                                   ulonglong fuzzydate,
+                                   const TABLE_SHARE *s, const char *field_name)
 {
   const ErrConvDecimal str(value);
   ulonglong nr;
   ulong sec_part;
   bool neg= my_decimal2seconds(value, &nr, &sec_part);
   return number_to_time_with_warn(neg, nr, sec_part, ltime, fuzzydate, &str,
-                                  field_name);
+                                  s, field_name);
 }
 
 
 bool int_to_datetime_with_warn(bool neg, ulonglong value, MYSQL_TIME *ltime,
-                               ulonglong fuzzydate, const char *field_name)
+                               ulonglong fuzzydate,
+                               const TABLE_SHARE *s, const char *field_name)
 {
-  const ErrConvInteger str(neg ? -value : value, !neg);
+  const ErrConvInteger str(neg ? - (longlong) value : (longlong) value, !neg);
   return number_to_time_with_warn(neg, value, 0, ltime,
-                                  fuzzydate, &str, field_name);
+                                  fuzzydate, &str, s, field_name);
 }
 
 
@@ -477,6 +552,7 @@ void localtime_to_TIME(MYSQL_TIME *to, struct tm *from)
   to->second=   (int) from->tm_sec;
 }
 
+
 void calc_time_from_sec(MYSQL_TIME *to, long seconds, long microseconds)
 {
   long t_seconds;
@@ -546,8 +622,7 @@ bool parse_date_time_format(timestamp_type format_type,
   {
     if (*ptr == '%' && ptr+1 != end)
     {
-      uint position;
-      LINT_INIT(position);
+      uint UNINIT_VAR(position);
       switch (*++ptr) {
       case 'y':					// Year
       case 'Y':
@@ -761,7 +836,7 @@ DATE_TIME_FORMAT
       !parse_date_time_format(format_type, format_str,
 			      format_length, &tmp))
   {
-    tmp.format.str=    (char*) format_str;
+    tmp.format.str=    format_str;
     tmp.format.length= format_length;
     return date_time_format_copy((THD *)0, &tmp);
   }
@@ -788,7 +863,8 @@ DATE_TIME_FORMAT
 DATE_TIME_FORMAT *date_time_format_copy(THD *thd, DATE_TIME_FORMAT *format)
 {
   DATE_TIME_FORMAT *new_format;
-  ulong length= sizeof(*format) + format->format.length + 1;
+  size_t length= sizeof(*format) + format->format.length + 1;
+  char *format_pos;
 
   if (thd)
     new_format= (DATE_TIME_FORMAT *) thd->alloc(length);
@@ -797,14 +873,13 @@ DATE_TIME_FORMAT *date_time_format_copy(THD *thd, DATE_TIME_FORMAT *format)
   if (new_format)
   {
     /* Put format string after current pos */
-    new_format->format.str= (char*) (new_format+1);
+    new_format->format.str= format_pos= (char*) (new_format+1);
     memcpy((char*) new_format->positions, (char*) format->positions,
 	   sizeof(format->positions));
     new_format->time_separator= format->time_separator;
     /* We make the string null terminated for easy printf in SHOW VARIABLES */
-    memcpy((char*) new_format->format.str, format->format.str,
-	   format->format.length);
-    new_format->format.str[format->format.length]= 0;
+    memcpy(format_pos, format->format.str, format->format.length);
+    format_pos[format->format.length]= 0;
     new_format->format.length= format->format.length;
   }
   return new_format;
@@ -856,9 +931,10 @@ bool my_TIME_to_str(const MYSQL_TIME *ltime, String *str, uint dec)
 
 
 void make_truncated_value_warning(THD *thd,
-                                  MYSQL_ERROR::enum_warning_level level,
+                                  Sql_condition::enum_warning_level level,
                                   const ErrConv *sval,
 				  timestamp_type time_type,
+                                  const char *db_name, const char *table_name,
                                   const char *field_name)
 {
   char warn_buff[MYSQL_ERRMSG_SIZE];
@@ -878,19 +954,27 @@ void make_truncated_value_warning(THD *thd,
       break;
   }
   if (field_name)
+  {
+    if (!db_name)
+      db_name= "";
+    if (!table_name)
+      table_name= "";
+
     cs->cset->snprintf(cs, warn_buff, sizeof(warn_buff),
-                       ER(ER_TRUNCATED_WRONG_VALUE_FOR_FIELD),
-                       type_str, sval->ptr(), field_name,
-                       (ulong) thd->warning_info->current_row_for_warning());
+                       ER_THD(thd, ER_TRUNCATED_WRONG_VALUE_FOR_FIELD),
+                       type_str, sval->ptr(),
+                       db_name, table_name, field_name,
+                       (ulong) thd->get_stmt_da()->current_row_for_warning());
+  }
   else
   {
     if (time_type > MYSQL_TIMESTAMP_ERROR)
       cs->cset->snprintf(cs, warn_buff, sizeof(warn_buff),
-                         ER(ER_TRUNCATED_WRONG_VALUE),
+                         ER_THD(thd, ER_TRUNCATED_WRONG_VALUE),
                          type_str, sval->ptr());
     else
       cs->cset->snprintf(cs, warn_buff, sizeof(warn_buff),
-                         ER(ER_WRONG_VALUE), type_str, sval->ptr());
+                         ER_THD(thd, ER_WRONG_VALUE), type_str, sval->ptr());
   }
   push_warning(thd, level,
                ER_TRUNCATED_WRONG_VALUE, warn_buff);
@@ -905,11 +989,11 @@ void make_truncated_value_warning(THD *thd,
 #define GET_PART(X, N) X % N ## LL; X/= N ## LL
 
 bool date_add_interval(MYSQL_TIME *ltime, interval_type int_type,
-                       INTERVAL interval)
+                       const INTERVAL &interval)
 {
   long period, sign;
 
-  sign= (interval.neg == ltime->neg ? 1 : -1);
+  sign= (interval.neg == (bool)ltime->neg ? 1 : -1);
 
   switch (int_type) {
   case INTERVAL_SECOND:
@@ -932,6 +1016,12 @@ bool date_add_interval(MYSQL_TIME *ltime, interval_type int_type,
     my_bool neg= 0;
     enum enum_mysql_timestamp_type time_type= ltime->time_type;
 
+    if (((ulonglong) interval.day +
+         (ulonglong) interval.hour / 24 +
+         (ulonglong) interval.minute / 24 / 60 +
+         (ulonglong) interval.second / 24 / 60 / 60) > MAX_DAY_NUMBER)
+      goto invalid_date;
+
     if (time_type != MYSQL_TIMESTAMP_TIME)
       ltime->day+= calc_daynr(ltime->year, ltime->month, 1) - 1;
 
@@ -956,6 +1046,8 @@ bool date_add_interval(MYSQL_TIME *ltime, interval_type int_type,
       ltime->day= 0;
       return 0;
     }
+    else if (ltime->neg)
+      goto invalid_date;
 
     if (int_type != INTERVAL_DAY)
       ltime->time_type= MYSQL_TIMESTAMP_DATETIME; // Return full date
@@ -1010,11 +1102,14 @@ bool date_add_interval(MYSQL_TIME *ltime, interval_type int_type,
     return 0;                                   // Ok
 
 invalid_date:
-  push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                      ER_DATETIME_FUNCTION_OVERFLOW,
-                      ER(ER_DATETIME_FUNCTION_OVERFLOW),
-                      ltime->time_type == MYSQL_TIMESTAMP_TIME ?
-                      "time" : "datetime");
+  {
+    THD *thd= current_thd;
+    push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                        ER_DATETIME_FUNCTION_OVERFLOW,
+                        ER_THD(thd, ER_DATETIME_FUNCTION_OVERFLOW),
+                        ltime->time_type == MYSQL_TIMESTAMP_TIME ?
+                        "time" : "datetime");
+  }
 null_date:
   return 1;
 }
@@ -1047,8 +1142,8 @@ null_date:
 */
 
 bool
-calc_time_diff(MYSQL_TIME *l_time1, MYSQL_TIME *l_time2, int l_sign, longlong *seconds_out,
-               long *microseconds_out)
+calc_time_diff(const MYSQL_TIME *l_time1, const MYSQL_TIME *l_time2,
+               int l_sign, longlong *seconds_out, long *microseconds_out)
 {
   long days;
   bool neg;
@@ -1074,13 +1169,13 @@ calc_time_diff(MYSQL_TIME *l_time1, MYSQL_TIME *l_time2, int l_sign, longlong *s
 			       (uint) l_time2->day);
   }
 
-  microseconds= ((longlong)days*LL(86400) +
+  microseconds= ((longlong)days * SECONDS_IN_24H +
                  (longlong)(l_time1->hour*3600L +
                             l_time1->minute*60L +
                             l_time1->second) -
                  l_sign*(longlong)(l_time2->hour*3600L +
                                    l_time2->minute*60L +
-                                   l_time2->second)) * LL(1000000) +
+                                   l_time2->second)) * 1000000LL +
                 (longlong)l_time1->second_part -
                 l_sign*(longlong)l_time2->second_part;
 
@@ -1096,6 +1191,35 @@ calc_time_diff(MYSQL_TIME *l_time1, MYSQL_TIME *l_time2, int l_sign, longlong *s
 }
 
 
+bool calc_time_diff(const MYSQL_TIME *l_time1, const MYSQL_TIME *l_time2,
+                    int l_sign, MYSQL_TIME *l_time3, ulonglong fuzzydate)
+{
+  longlong seconds;
+  long microseconds;
+  bzero((char *) l_time3, sizeof(*l_time3));
+  l_time3->neg= calc_time_diff(l_time1, l_time2, l_sign,
+			       &seconds, &microseconds);
+  /*
+    For MYSQL_TIMESTAMP_TIME only:
+      If first argument was negative and diff between arguments
+      is non-zero we need to swap sign to get proper result.
+  */
+  if (l_time1->neg && (seconds || microseconds))
+    l_time3->neg= 1 - l_time3->neg;         // Swap sign of result
+
+  /*
+    seconds is longlong, when casted to long it may become a small number
+    even if the original seconds value was too large and invalid.
+    as a workaround we limit seconds by a large invalid long number
+    ("invalid" means > TIME_MAX_SECOND)
+  */
+  set_if_smaller(seconds, INT_MAX32);
+  calc_time_from_sec(l_time3, (long) seconds, microseconds);
+  return ((fuzzydate & TIME_NO_ZERO_DATE) && (seconds == 0) &&
+          (microseconds == 0));
+}
+
+
 /*
   Compares 2 MYSQL_TIME structures
 
@@ -1112,7 +1236,7 @@ calc_time_diff(MYSQL_TIME *l_time1, MYSQL_TIME *l_time2, int l_sign, longlong *s
 
 */
 
-int my_time_compare(MYSQL_TIME *a, MYSQL_TIME *b)
+int my_time_compare(const MYSQL_TIME *a, const MYSQL_TIME *b)
 {
   ulonglong a_t= pack_time(a);
   ulonglong b_t= pack_time(b);
@@ -1167,8 +1291,8 @@ make_date_with_warn(MYSQL_TIME *ltime, ulonglong fuzzy_date,
   {
     /* e.g. negative time */
     ErrConvTime str(ltime);
-    make_truncated_value_warning(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                                 &str, ts_type, 0);
+    make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
+                                 &str, ts_type, NULL, NULL, NULL);
     return true;
   }
   if ((ltime->time_type= ts_type) == MYSQL_TIMESTAMP_DATE)
@@ -1195,3 +1319,208 @@ void time_to_daytime_interval(MYSQL_TIME *ltime)
   ltime->hour%= 24;
   ltime->time_type= MYSQL_TIMESTAMP_NONE;
 }
+
+
+/*** Conversion from TIME to DATETIME ***/
+
+/*
+  Simple case: TIME is within normal 24 hours internal.
+  Mix DATE part of ldate and TIME part of ltime together.
+*/
+static void
+mix_date_and_time_simple(MYSQL_TIME *ldate, const MYSQL_TIME *ltime)
+{
+  DBUG_ASSERT(ldate->time_type == MYSQL_TIMESTAMP_DATE ||
+              ldate->time_type == MYSQL_TIMESTAMP_DATETIME);
+  ldate->hour= ltime->hour;
+  ldate->minute= ltime->minute;
+  ldate->second= ltime->second;
+  ldate->second_part= ltime->second_part;
+  ldate->time_type= MYSQL_TIMESTAMP_DATETIME;
+}
+
+
+/*
+  Complex case: TIME is negative or outside of the 24 hour interval.
+*/
+static void
+mix_date_and_time_complex(MYSQL_TIME *ldate, const MYSQL_TIME *ltime)
+{
+  DBUG_ASSERT(ldate->time_type == MYSQL_TIMESTAMP_DATE ||
+              ldate->time_type == MYSQL_TIMESTAMP_DATETIME);
+  longlong seconds;
+  long days, useconds;
+  int sign= ltime->neg ? 1 : -1;
+  ldate->neg= calc_time_diff(ldate, ltime, sign, &seconds, &useconds);
+
+  DBUG_ASSERT(!ldate->neg);
+  DBUG_ASSERT(ldate->year > 0);
+
+  days= (long) (seconds / SECONDS_IN_24H);
+  calc_time_from_sec(ldate, seconds % SECONDS_IN_24H, useconds);
+  get_date_from_daynr(days, &ldate->year, &ldate->month, &ldate->day);
+  ldate->time_type= MYSQL_TIMESTAMP_DATETIME;
+}
+
+
+/**
+  Mix a date value and a time value.
+
+  @param  IN/OUT  ldate  Date value.
+  @param          ltime  Time value.
+*/
+static void
+mix_date_and_time(MYSQL_TIME *to, const MYSQL_TIME *from)
+{
+  if (!from->neg && from->hour < 24)
+    mix_date_and_time_simple(to, from);
+  else
+    mix_date_and_time_complex(to, from);
+}
+
+
+/**
+  Get current date in DATE format
+*/
+void set_current_date(THD *thd, MYSQL_TIME *to)
+{
+  thd->variables.time_zone->gmt_sec_to_TIME(to, thd->query_start());
+  thd->time_zone_used= 1;
+  datetime_to_date(to);
+}
+
+
+/**
+  5.5 compatible conversion from TIME to DATETIME
+*/
+static bool
+time_to_datetime_old(THD *thd, const MYSQL_TIME *from, MYSQL_TIME *to)
+{
+  DBUG_ASSERT(from->time_type == MYSQL_TIMESTAMP_TIME);
+
+  if (from->neg)
+    return true;
+
+  /* Set the date part */
+  uint day= from->hour / 24;
+  to->day= day % 31;
+  to->month= day / 31;
+  to->year= 0;
+  /* Set the time part */
+  to->hour= from->hour % 24;
+  to->minute= from->minute;
+  to->second= from->second;
+  to->second_part= from->second_part;
+  /* set sign and type */
+  to->neg= 0;
+  to->time_type= MYSQL_TIMESTAMP_DATETIME;
+  return false;
+}
+
+
+/**
+  Convert time to datetime.
+
+  The time value is added to the current datetime value.
+  @param  IN  ltime    Time value to convert from.
+  @param  OUT ltime2   Datetime value to convert to.
+*/
+bool
+time_to_datetime(THD *thd, const MYSQL_TIME *from, MYSQL_TIME *to)
+{
+  if (thd->variables.old_behavior & OLD_MODE_ZERO_DATE_TIME_CAST)
+    return time_to_datetime_old(thd, from, to);
+  set_current_date(thd, to);
+  mix_date_and_time(to, from);
+  return false;
+}
+
+
+bool
+time_to_datetime_with_warn(THD *thd,
+                           const MYSQL_TIME *from, MYSQL_TIME *to,
+                           ulonglong fuzzydate)
+{
+  int warn= 0;
+  DBUG_ASSERT(from->time_type == MYSQL_TIMESTAMP_TIME);
+  /*
+    After time_to_datetime() we need to do check_date(), as
+    the caller may want TIME_NO_ZERO_DATE or TIME_NO_ZERO_IN_DATE.
+    Note, the SQL standard time->datetime conversion mode always returns
+    a valid date based on CURRENT_DATE. So we need to do check_date()
+    only in the old mode.
+  */
+  if (time_to_datetime(thd, from, to) ||
+      ((thd->variables.old_behavior & OLD_MODE_ZERO_DATE_TIME_CAST) &&
+        check_date(to, fuzzydate, &warn)))
+  {
+    ErrConvTime str(from);
+    make_truncated_value_warning(thd, Sql_condition::WARN_LEVEL_WARN,
+                                 &str, MYSQL_TIMESTAMP_DATETIME, NULL, NULL, NULL); 
+    return true;
+  }
+  return false;
+}
+
+
+bool datetime_to_time_with_warn(THD *thd, const MYSQL_TIME *dt,
+                                MYSQL_TIME *tm, uint dec)
+{
+  if (thd->variables.old_behavior & OLD_MODE_ZERO_DATE_TIME_CAST)
+  {
+    *tm= *dt;
+    datetime_to_time(tm);
+    return false;
+  }
+  else /* new mode */
+  {
+    MYSQL_TIME current_date;
+    set_current_date(thd, &current_date);
+    calc_time_diff(dt, &current_date, 1, tm, 0);
+  }
+  int warnings= 0;
+  return check_time_range(tm, dec, &warnings);
+}
+
+
+longlong pack_time(const MYSQL_TIME *my_time)
+{
+  return  ((((((my_time->year     * 13ULL +
+               my_time->month)    * 32ULL +
+               my_time->day)      * 24ULL +
+               my_time->hour)     * 60ULL +
+               my_time->minute)   * 60ULL +
+               my_time->second)   * 1000000ULL +
+               my_time->second_part) * (my_time->neg ? -1 : 1);
+}
+
+#define get_one(WHERE, FACTOR) WHERE= (ulong)(packed % FACTOR); packed/= FACTOR
+
+void unpack_time(longlong packed, MYSQL_TIME *my_time,
+                     enum_mysql_timestamp_type ts_type)
+{
+  if ((my_time->neg= packed < 0))
+    packed= -packed;
+  get_one(my_time->second_part, 1000000ULL);
+  get_one(my_time->second,           60U);
+  get_one(my_time->minute,           60U);
+  get_one(my_time->hour,             24U);
+  get_one(my_time->day,              32U);
+  get_one(my_time->month,            13U);
+  my_time->year= (uint)packed;
+  my_time->time_type= ts_type;
+  switch (ts_type) {
+  case MYSQL_TIMESTAMP_TIME:
+    my_time->hour+= (my_time->month * 32 + my_time->day) * 24;
+    my_time->month= my_time->day= 0;
+    break;
+  case MYSQL_TIMESTAMP_DATE:
+    my_time->hour= my_time->minute= my_time->second= my_time->second_part= 0;
+    break;
+  case MYSQL_TIMESTAMP_NONE:
+  case MYSQL_TIMESTAMP_ERROR:
+    DBUG_ASSERT(0);
+  case MYSQL_TIMESTAMP_DATETIME:
+    break;
+  }
+}
