@@ -17,21 +17,21 @@
 /*  Include relevant MariaDB header file.                  */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <io.h>
 #include <fcntl.h>
 #if defined(__BORLANDC__)
 #define __MFC_COMPAT__                   // To define min/max as macro
 #endif
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !_WIN32
 #if defined(UNIX)
 #include <errno.h>
 #else   // !UNIX
 #include <io.h>
 #endif
 #include <fcntl.h>
-#endif  // !__WIN__
+#endif  // !_WIN32
 
 /***********************************************************************/
 /*  Include application header files:                                  */
@@ -89,11 +89,11 @@ int GZFAM::Zerror(PGLOBAL g)
   strcpy(g->Message, gzerror(Zfile, &errnum));
 
   if (errnum == Z_ERRNO)
-#if defined(__WIN__)
-    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(NULL));
-#else   // !__WIN__
-    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
-#endif  // !__WIN__
+#if defined(_WIN32)
+    snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(NULL));
+#else   // !_WIN32
+    snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
+#endif  // !_WIN32
 
     return (errnum == Z_STREAM_END) ? RC_EF : RC_FX;
   } // end of Zerror
@@ -152,7 +152,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
 //      Last = Nrec;              // For ZBKFAM
         Tdbp->ResetSize();
       } else {
-        sprintf(g->Message, MSG(NO_PART_DEL), "GZ");
+        snprintf(g->Message, sizeof(g->Message), MSG(NO_PART_DEL), "GZ");
         return true;
       } // endif filter
 
@@ -161,7 +161,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
       strcpy(opmode, "a+");
       break;
     default:
-      sprintf(g->Message, MSG(BAD_OPEN_MODE), mode);
+      snprintf(g->Message, sizeof(g->Message), MSG(BAD_OPEN_MODE), mode);
       return true;
     } // endswitch Mode
 
@@ -174,7 +174,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
   Zfile = gzopen(PlugSetPath(filename, To_File, Tdbp->GetPath()), opmode);
 
   if (Zfile == NULL) {
-    sprintf(g->Message, MSG(GZOPEN_ERROR),
+    snprintf(g->Message, sizeof(g->Message), MSG(GZOPEN_ERROR),
             opmode, (int)errno, filename);
     strcat(strcat(g->Message, ": "), strerror(errno));
     return (mode == MODE_READ && errno == ENOENT)
@@ -249,13 +249,13 @@ int GZFAM::GetNextPos(void)
 /***********************************************************************/
 bool GZFAM::SetPos(PGLOBAL g, int pos __attribute__((unused)))
   {
-  sprintf(g->Message, MSG(NO_SETPOS_YET), "GZ");
+  snprintf(g->Message, sizeof(g->Message), MSG(NO_SETPOS_YET), "GZ");
   return true;
 #if 0
   Fpos = pos;
 
   if (fseek(Stream, Fpos, SEEK_SET)) {
-    sprintf(g->Message, MSG(FSETPOS_ERROR), Fpos);
+    snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), Fpos);
     return true;
     } // endif
 
@@ -655,7 +655,7 @@ int ZBKFAM::DeleteRecords(PGLOBAL g, int irc)
 
     if (!defp->SetIntCatInfo("Blocks", 0) ||
         !defp->SetIntCatInfo("Last", 0)) {
-      sprintf(g->Message, MSG(UPDATE_ERROR), "Header");
+      snprintf(g->Message, sizeof(g->Message), MSG(UPDATE_ERROR), "Header");
       return RC_FX;
     } else
       return RC_OK;
@@ -764,9 +764,9 @@ bool GZXFAM::AllocateBuffer(PGLOBAL g)
     if (Tdbp->GetFtype() < 2)
       // if not binary, the file is physically a text file
       for (int len = Lrecl; len <= Buflen; len += Lrecl) {
-#if defined(__WIN__)
+#if defined(_WIN32)
         To_Buf[len - 2] = '\r';
-#endif   // __WIN__
+#endif   // _WIN32
         To_Buf[len - 1] = '\n';
         } // endfor len
 
@@ -932,7 +932,7 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
 #if defined(NOLIB)
   if (!zlib && LoadZlib()) {
-    sprintf(g->Message, MSG(DLL_LOAD_ERROR), GetLastError(), "zlib.dll");
+    snprintf(g->Message, sizeof(g->Message), MSG(DLL_LOAD_ERROR), GetLastError(), "zlib.dll");
     return TRUE;
     } // endif zlib
 #endif
@@ -964,9 +964,9 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
   if (zrc != Z_OK) {
     if (Zstream->msg)
-      sprintf(g->Message, "%s error: %s", msg, Zstream->msg);
+      snprintf(g->Message, sizeof(g->Message), "%s error: %s", msg, Zstream->msg);
     else
-      sprintf(g->Message, "%s error: %d", msg, zrc);
+      snprintf(g->Message, sizeof(g->Message), "%s error: %d", msg, zrc);
 
     return TRUE;
     } // endif zrc
@@ -1017,9 +1017,9 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
         return FALSE;
       case RC_FX:
 #if defined(UNIX)
-        sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
+        snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
 #else
-        sprintf(g->Message, MSG(READ_ERROR), To_File, _strerror(NULL));
+        snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, _strerror(NULL));
 #endif
       case RC_NF:
         return TRUE;
@@ -1027,7 +1027,7 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
     // Some old tables can have PlugDB in their header
     if (strcmp(To_Buf, "PlugDB")) {
-      sprintf(g->Message, MSG(BAD_HEADER), Tdbp->GetFile(g));
+      snprintf(g->Message, sizeof(g->Message), MSG(BAD_HEADER), Tdbp->GetFile(g));
       return TRUE;
       } // endif strcmp
 
@@ -1062,7 +1062,7 @@ int ZLBFAM::GetNextPos(void)
 /***********************************************************************/
 bool ZLBFAM::SetPos(PGLOBAL g, int pos __attribute__((unused)))
   {
-  sprintf(g->Message, MSG(NO_SETPOS_YET), "GZ");
+  snprintf(g->Message, sizeof(g->Message), MSG(NO_SETPOS_YET), "GZ");
   return true;
 #if 0 // All this must be checked
   if (pos < 0) {
@@ -1145,7 +1145,7 @@ int ZLBFAM::ReadBuffer(PGLOBAL g)
     // fseek is required only in non sequential reading
     if (CurBlk != OldBlk + 1)
       if (fseek(Stream, Fpos, SEEK_SET)) {
-        sprintf(g->Message, MSG(FSETPOS_ERROR), Fpos);
+        snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), Fpos);
         return RC_FX;
         } // endif fseek
 
@@ -1207,9 +1207,9 @@ int ZLBFAM::ReadBuffer(PGLOBAL g)
 
  err:
 #if defined(UNIX)
-  sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
+  snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
 #else
-  sprintf(g->Message, MSG(READ_ERROR), To_File, _strerror(NULL));
+  snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, _strerror(NULL));
 #endif
   return RC_FX;
   } // end of ReadBuffer
@@ -1225,7 +1225,7 @@ int ZLBFAM::ReadCompressedBuffer(PGLOBAL g, void *rdbuf)
     num_read++;
 
     if (Optimized && BlkLen != signed(*Zlenp + sizeof(int))) {
-      sprintf(g->Message, MSG(BAD_BLK_SIZE), CurBlk + 1);
+      snprintf(g->Message, sizeof(g->Message), MSG(BAD_BLK_SIZE), CurBlk + 1);
       return RC_NF;
       } // endif BlkLen
 
@@ -1238,9 +1238,9 @@ int ZLBFAM::ReadCompressedBuffer(PGLOBAL g, void *rdbuf)
 
     if (zrc != Z_OK) {
       if (Zstream->msg)
-        sprintf(g->Message, MSG(FUNC_ERR_S), "inflate", Zstream->msg);
+        snprintf(g->Message, sizeof(g->Message), MSG(FUNC_ERR_S), "inflate", Zstream->msg);
       else
-        sprintf(g->Message, MSG(FUNCTION_ERROR), "inflate", (int)zrc);
+        snprintf(g->Message, sizeof(g->Message), MSG(FUNCTION_ERROR), "inflate", (int)zrc);
 
       return RC_NF;
       } // endif zrc
@@ -1328,9 +1328,9 @@ bool ZLBFAM::WriteCompressedBuffer(PGLOBAL g)
 
   if (zrc != Z_OK) {
     if (Zstream->msg)
-      sprintf(g->Message, MSG(FUNC_ERR_S), "deflate", Zstream->msg);
+      snprintf(g->Message, sizeof(g->Message), MSG(FUNC_ERR_S), "deflate", Zstream->msg);
     else
-      sprintf(g->Message, MSG(FUNCTION_ERROR), "deflate", (int)zrc);
+      snprintf(g->Message, sizeof(g->Message), MSG(FUNCTION_ERROR), "deflate", (int)zrc);
 
     return TRUE;
   } else
@@ -1340,7 +1340,7 @@ bool ZLBFAM::WriteCompressedBuffer(PGLOBAL g)
   BlkLen = *Zlenp + sizeof(int);
 
   if (fwrite(Zlenp, 1, BlkLen, Stream) != (size_t)BlkLen) {
-    sprintf(g->Message, MSG(FWRITE_ERROR), strerror(errno));
+    snprintf(g->Message, sizeof(g->Message), MSG(FWRITE_ERROR), strerror(errno));
     return TRUE;
     } // endif size
 
