@@ -6,19 +6,19 @@
 /*  This file contains the JDBC connection classes functions.          */
 /***********************************************************************/
 
-#if defined(__WIN__)
+#if defined(_WIN32)
 // This is needed for RegGetValue
 #define _WINVER 0x0601
 #undef  _WIN32_WINNT
 #define _WIN32_WINNT 0x0601
-#endif   // __WIN__
+#endif   // _WIN32
 
 /***********************************************************************/
 /*  Include relevant MariaDB header file.                              */
 /***********************************************************************/
 #include <my_global.h>
 #include <m_string.h>
-#if defined(__WIN__)
+#if defined(_WIN32)
 //nclude <io.h>
 //nclude <fcntl.h>
 #include <direct.h>                      // for getcwd
@@ -26,7 +26,7 @@
 #define __MFC_COMPAT__                   // To define min/max as macro
 #endif   // __BORLANDC__
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !_WIN32
 #if defined(UNIX)
 #include <errno.h>
 #else   // !UNIX
@@ -36,7 +36,7 @@
 #include <stdlib.h>                      // for getenv
 //nclude <fcntl.h>
 #define NODW
-#endif  // !__WIN__
+#endif  // !_WIN32
 
 /***********************************************************************/
 /*  Required objects includes.                                         */
@@ -53,9 +53,9 @@
 #include "osutil.h"
 
 
-//#if defined(__WIN__)
+//#if defined(_WIN32)
 //extern "C" HINSTANCE s_hModule;           // Saved module handle
-//#endif   // __WIN__
+//#endif   // _WIN32
 #define nullptr 0
 
 TYPCONV GetTypeConv();
@@ -178,30 +178,38 @@ int TranslateJDBCType(int stp, char *tn, int prec, int& len, char& v)
 				v = 'Y';
 			}	// endif len
 
-		break;
-	case 92:   // TIME
-		type = TYPE_DATE;
-		len = 8 + ((prec) ? (prec+1) : 0);
-		v = 'T';
-		break;
-	case -5:   // BIGINT
-		type = TYPE_BIGINT;
-		break;
-	case 0:    // NULL
-	case -2:   // BINARY
-	case -4:   // LONGVARBINARY
-	case 70:   // DATALINK
-	case 2000: // JAVA_OBJECT
-	case 2001: // DISTINCT
-	case 2002: // STRUCT
-	case 2003: // ARRAY
-	case 2004: // BLOB
-	case 2005: // CLOB
-	case 2006: // REF
-	case 2009: // SQLXML
-	case 2011: // NCLOB
-	default:
-		type = TYPE_ERROR;
+			break;
+		case 92:   // TIME
+			type = TYPE_DATE;
+			len = 8 + ((prec) ? (prec + 1) : 0);
+			v = 'T';
+			break;
+		case -5:   // BIGINT
+			type = TYPE_BIGINT;
+			break;
+		case 1111: // UNKNOWN or UUID
+			if (!tn || !stricmp(tn, "UUID")) {
+				type = TYPE_STRING;
+				len = 36;
+				break;
+			}	// endif tn
+
+			// Pass through
+		case 0:    // NULL
+		case -2:   // BINARY
+		case -4:   // LONGVARBINARY
+		case 70:   // DATALINK
+		case 2000: // JAVA_OBJECT
+		case 2001: // DISTINCT
+		case 2002: // STRUCT
+		case 2003: // ARRAY
+		case 2004: // BLOB
+		case 2005: // CLOB
+		case 2006: // REF
+		case 2009: // SQLXML
+		case 2011: // NCLOB
+		default:
+			type = TYPE_ERROR;
 		len = 0;
 	} // endswitch type
 
@@ -681,17 +689,17 @@ bool JDBConn::SetUUID(PGLOBAL g, PTDBJDBC tjp)
 		ncol = env->CallIntMethod(job, catid, parms);
 
 		if (Check(ncol)) {
-			sprintf(g->Message, "%s: %s", fnc, Msg);
+			snprintf(g->Message, sizeof(g->Message), "%s: %s", fnc, Msg);
 			goto err;
 		}	// endif Check
 
 		rc = env->CallBooleanMethod(job, readid);
 
 		if (Check(rc)) {
-			sprintf(g->Message, "ReadNext: %s", Msg);
+			snprintf(g->Message, sizeof(g->Message), "ReadNext: %s", Msg);
 			goto err;
 		} else if (rc == 0) {
-			sprintf(g->Message, "table %s does not exist", tjp->TableName);
+			snprintf(g->Message, sizeof(g->Message), "table %s does not exist", tjp->TableName);
 			goto err;
 		}	// endif rc
 
@@ -699,7 +707,7 @@ bool JDBConn::SetUUID(PGLOBAL g, PTDBJDBC tjp)
 		ctyp = (int)env->CallIntMethod(job, intfldid, 5, nullptr);
 
 		//if (Check((ctyp == 666) ? -1 : 1)) {
-		//	sprintf(g->Message, "Getting ctyp: %s", Msg);
+		//	snprintf(g->Message, sizeof(g->Message), "Getting ctyp: %s", Msg);
 		//	goto err;
 		//} // endif ctyp
 
@@ -807,7 +815,7 @@ bool JDBConn::Connect(PJPARM sop)
 	env->DeleteLocalRef(parms);				 	// Not used anymore
 
 	if (err) {
-		sprintf(g->Message, "Connecting: %s rc=%d", Msg, (int)rc);
+		snprintf(g->Message, sizeof(g->Message), "Connecting: %s rc=%d", Msg, (int)rc);
 		return true;
 	}	// endif Msg
 
@@ -855,14 +863,14 @@ int JDBConn::ExecuteCommand(PCSZ sql)
 	env->DeleteLocalRef(qry);
 
 	if (Check(n)) {
-		sprintf(g->Message, "Execute: %s", Msg);
+		snprintf(g->Message, sizeof(g->Message), "Execute: %s", Msg);
 		return RC_FX;
 	} // endif n
 
 	m_Ncol = env->CallIntMethod(job, grs);
 
 	if (Check(m_Ncol)) {
-		sprintf(g->Message, "GetResult: %s", Msg);
+		snprintf(g->Message, sizeof(g->Message), "GetResult: %s", Msg);
 		rc = RC_FX;
 	} else if (m_Ncol) {
 		strcpy(g->Message, "Result set column number");
@@ -911,7 +919,7 @@ int JDBConn::Fetch(int pos)
 
 			m_Rows += (int)rc;
 		} else
-			sprintf(g->Message, "Fetch: %s", Msg);
+			snprintf(g->Message, sizeof(g->Message), "Fetch: %s", Msg);
 
 	} // endif pos
 
@@ -953,7 +961,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 
 	if (rank == 0)
 		if (!name || (jn = env->NewStringUTF(name)) == nullptr) {
-			sprintf(g->Message, "Fail to allocate jstring %s", SVP(name));
+			snprintf(g->Message, sizeof(g->Message), "Fail to allocate jstring %s", SVP(name));
 			throw (int)TYPE_AM_JDBC;
 		}	// endif name
 
@@ -961,7 +969,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 	ctyp = env->CallIntMethod(job, typid, rank, jn);
 
 	if (Check((ctyp == 666) ? -1 : 1)) {
-		sprintf(g->Message, "Getting ctyp: %s", Msg);
+		snprintf(g->Message, sizeof(g->Message), "Getting ctyp: %s", Msg);
 		throw (int)TYPE_AM_JDBC;
 	} // endif Check
 
@@ -970,7 +978,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 			jb = env->CallObjectMethod(job, objfldid, (jint)rank, jn);
 
 			if (Check(0)) {
-				sprintf(g->Message, "Getting jp: %s", Msg);
+				snprintf(g->Message, sizeof(g->Message), "Getting jp: %s", Msg);
 				throw (int)TYPE_AM_JDBC;
 			} // endif Check
 
@@ -1090,7 +1098,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 		if (rank == 0)
 			env->DeleteLocalRef(jn);
 
-		sprintf(g->Message, "SetColumnValue: %s rank=%d ctyp=%d", Msg, rank, (int)ctyp);
+		snprintf(g->Message, sizeof(g->Message), "SetColumnValue: %s rank=%d ctyp=%d", Msg, rank, (int)ctyp);
 		throw (int)TYPE_AM_JDBC;
 	} // endif Check
 
@@ -1112,7 +1120,7 @@ bool JDBConn::PrepareSQL(PCSZ sql)
 		jstring qry = env->NewStringUTF(sql);
 
 		if (Check(env->CallBooleanMethod(job, prepid, qry)))
-			sprintf(g->Message, "CreatePrepStmt: %s", Msg);
+			snprintf(g->Message, sizeof(g->Message), "CreatePrepStmt: %s", Msg);
 		else
 			b = false;
 
@@ -1142,7 +1150,7 @@ int JDBConn::ExecuteQuery(PCSZ sql)
 			m_Aff = 0;			  // Affected rows
 			rc = RC_OK;
 		} else
-			sprintf(g->Message, "ExecuteQuery: %s", Msg);
+			snprintf(g->Message, sizeof(g->Message), "ExecuteQuery: %s", Msg);
 
 		env->DeleteLocalRef(qry);
 	} // endif xqid
@@ -1170,7 +1178,7 @@ int JDBConn::ExecuteUpdate(PCSZ sql)
 			m_Aff = (int)n;			  // Affected rows
 			rc = RC_OK;
 		} else
-			sprintf(g->Message, "ExecuteUpdate: %s n=%d", Msg, n);
+			snprintf(g->Message, sizeof(g->Message), "ExecuteUpdate: %s n=%d", Msg, n);
 
 		env->DeleteLocalRef(qry);
 	} // endif xuid
@@ -1188,9 +1196,14 @@ int JDBConn::GetResultSize(PCSZ sql, PCOL colp)
 	if ((rc = ExecuteQuery(sql)) != RC_OK)
 		return -1;
 
-	if ((rc = Fetch()) > 0)
-		SetColumnValue(1, NULL, colp->GetValue());
-	else
+	if ((rc = Fetch()) > 0) {
+		try {
+			SetColumnValue(1, NULL, colp->GetValue());
+		}	catch (...) {
+			return -4;
+		} // end catch
+
+	} else
 		return -2;
 
 	if ((rc = Fetch()) != 0)
@@ -1215,7 +1228,7 @@ int JDBConn::ExecuteSQL(void)
 		if (n == -3)
 			strcpy(g->Message, "SQL statement is not prepared");
 		else if (Check(n))
-			sprintf(g->Message, "ExecutePrep: %s", Msg);
+			snprintf(g->Message, sizeof(g->Message), "ExecutePrep: %s", Msg);
 		else {
 			m_Aff = (int)n;
 			rc = RC_OK;
@@ -1314,12 +1327,12 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 			env->CallVoidMethod(job, setid, i, datobj);
 			break;
 		default:
-			sprintf(g->Message, "Parm type %d not supported", val->GetType());
+			snprintf(g->Message, sizeof(g->Message), "Parm type %d not supported", val->GetType());
 			return true;
 		}	// endswitch Type
 
 	if (Check(jrc)) {
-		sprintf(g->Message, "SetParam: col=%s msg=%s", colp->GetName(), Msg);
+		snprintf(g->Message, sizeof(g->Message), "SetParam: col=%s msg=%s", colp->GetName(), Msg);
 		rc = true;
 	} // endif msg
 
@@ -1442,7 +1455,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 		for (i = 0; i < m_Ncol; i++) {
 			if (!(label = (jstring)env->CallObjectMethod(job, colid, i + 1, val))) {
 				if (Check())
-					sprintf(g->Message, "ColumnDesc: %s", Msg);
+					snprintf(g->Message, sizeof(g->Message), "ColumnDesc: %s", Msg);
 				else
 					strcpy(g->Message, "No result metadata");
 
@@ -1525,7 +1538,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 			break;
 #endif // 0
 		default:
-			sprintf(g->Message, "Invalid SQL function id");
+			snprintf(g->Message, sizeof(g->Message), "Invalid SQL function id");
 			return -1;
 		} // endswitch infotype
 
@@ -1536,7 +1549,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 		ncol = env->CallIntMethod(job, catid, parms);
 
 		if (Check(ncol)) {
-			sprintf(g->Message, "%s: %s", fnc, Msg);
+			snprintf(g->Message, sizeof(g->Message), "%s: %s", fnc, Msg);
 			env->DeleteLocalRef(parms);
 			return -1;
 		}	// endif Check
@@ -1561,7 +1574,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 		// Prepare retrieving column values
 		for (n = 0, crp = qrp->Colresp; crp; crp = crp->Next) {
 			if (!(tp = GetJDBCType(crp->Type))) {
-				sprintf(g->Message, MSG(INV_COLUMN_TYPE), crp->Type, crp->Name);
+				snprintf(g->Message, sizeof(g->Message), MSG(INV_COLUMN_TYPE), crp->Type, crp->Name);
 				return -1;
 			} // endif tp
 
@@ -1586,7 +1599,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 		// Now fetch the result
 		for (i = 0; i < qrp->Maxres; i++) {
 			if (Check(rc = Fetch(0))) {
-				sprintf(g->Message, "Fetch: %s", Msg);
+				snprintf(g->Message, sizeof(g->Message), "Fetch: %s", Msg);
 				return -1;
 			} if (rc == 0) {
         if (trace(1))
@@ -1655,7 +1668,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 
 				if (!(crp->Kdata = AllocValBlock(g, NULL, crp->Type, m_Rows,
 					crp->Clen, 0, FALSE, TRUE, uns))) {
-					sprintf(g->Message, MSG(INV_RESULT_TYPE),
+					snprintf(g->Message, sizeof(g->Message), MSG(INV_RESULT_TYPE),
 						GetFormatType(crp->Type));
 					return NULL;
 				} // endif Kdata
