@@ -1,11 +1,11 @@
 /************** MyConn C++ Program Source Code File (.CPP) **************/
 /* PROGRAM NAME: MYCONN                                                 */
 /* -------------                                                        */
-/*  Version 1.8                                                         */
+/*  Version 1.9                                                         */
 /*                                                                      */
 /* COPYRIGHT:                                                           */
 /* ----------                                                           */
-/*  (C) Copyright to the author Olivier BERTRAND          2007-2016     */
+/*  (C) Copyright to the author Olivier BERTRAND          2007-2017     */
 /*                                                                      */
 /* WHAT THIS PROGRAM DOES:                                              */
 /* -----------------------                                              */
@@ -35,11 +35,11 @@
 #include "my_sys.h"
 #include "mysqld_error.h"
 #endif   // !MYSQL_PREPARED_STATEMENTS
-#if defined(__WIN__)
+#if defined(_WIN32)
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !_WIN32
 #include "osutil.h"
-#endif  // !__WIN__
+#endif  // !_WIN32
 
 #include "global.h"
 #include "plgdbsem.h"
@@ -135,10 +135,13 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
                    FLD_KEY,  FLD_SCALE, FLD_RADIX,    FLD_NULL,
                    FLD_REM,  FLD_NO,    FLD_DEFAULT,  FLD_EXTRA,
                    FLD_CHARSET};
-  unsigned int length[] = {0, 4, 16, 4, 4, 4, 4, 4, 0, 0, 0, 0, 0};
-  char   *fld, *colname, *chset, *fmt, v, buf[128], uns[16], zero[16];
-  int     i, n, nf, ncol = sizeof(buftyp) / sizeof(int);
+  //unsigned int length[] = {0, 4, 16, 4, 4, 4, 4, 4, 0, 0, 0, 0, 0};
+	unsigned int length[] = {0, 4, 0, 4, 4, 4, 4, 4, 0, 0, 0, 0, 0};
+	PCSZ    fmt;
+	char   *fld, *colname, *chset, v, buf[128], uns[16], zero[16];
+  int     i, n, nf = 0, ncol = sizeof(buftyp) / sizeof(int);
   int     len, type, prec, rc, k = 0;
+	bool    b;
   PQRYRES qrp;
   PCOLRES crp;
   MYSQLC  myc;
@@ -157,7 +160,9 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
     /*  Do an evaluation of the result size.                            */
     /********************************************************************/
     STRING cmd(g, 64, "SHOW FULL COLUMNS FROM ");
-    bool   b = cmd.Append((PSZ)table);
+		b = cmd.Append('`');
+    b |= cmd.Append((PSZ)table);
+		b |= cmd.Append('`');
 
     b |= cmd.Append(" FROM ");
     b |= cmd.Append((PSZ)(db ? db : PlgGetUser(g)->DBName));
@@ -172,7 +177,7 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
       return NULL;
       } // endif b
 
-    if (trace)
+    if (trace(1))
       htrc("MyColumns: cmd='%s'\n", cmd.GetStr());
 
     if ((n = myc.GetResultSize(g, cmd.GetStr())) < 0) {
@@ -232,11 +237,31 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
     fld = myc.GetCharField(1);
     prec = 0;
     len = 0;
-    v = (chset && !strcmp(chset, "binary")) ? 'B' : 0;
+//  v = (chset && !strcmp(chset, "binary")) ? 'B' : 0;
+		v = 0;
     *uns = 0;
     *zero = 0;
+		b = false;
 
-    switch ((nf = sscanf(fld, "%[^(](%d,%d", buf, &len, &prec))) {
+		if (!strnicmp(fld, "enum", 4)) {
+			char *p2, *p1 = fld + 6;            // to skip enum('
+
+			while (true) {
+				p2 = strchr(p1, '\'');
+				len = MY_MAX(len, (int)(p2 - p1));
+				if (*++p2 != ',') break;
+				p1 = p2 + 2;
+			} // endwhile
+
+			v = (len > 255) ? 'V' : 0;
+			strcpy(buf, "enum");
+			b = true;
+		} else if (!strnicmp(fld, "set", 3)) {
+			len = (int)strlen(fld) - 2;
+			v = 'V';
+			strcpy(buf, "set");
+			b = true;
+		} else switch ((nf = sscanf(fld, "%[^(](%d,%d", buf, &len, &prec))) {
       case 3:
         nf = sscanf(fld, "%[^(](%d,%d) %s %s", buf, &len, &prec, uns, zero);
         break;
@@ -247,7 +272,7 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
         nf = sscanf(fld, "%s %s %s", buf, uns, zero) + 2;
         break;
       default:
-        sprintf(g->Message, MSG(BAD_FIELD_TYPE), fld);
+        snprintf(g->Message, sizeof(g->Message), MSG(BAD_FIELD_TYPE), fld);
         myc.Close();
         return NULL;
       } // endswitch nf
@@ -255,23 +280,23 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
     if ((type = MYSQLtoPLG(buf, &v)) == TYPE_ERROR) {
       if (v == 'K') {
         // Skip this column
-        sprintf(g->Message, "Column %s skipped (unsupported type %s)",
+        snprintf(g->Message, sizeof(g->Message), "Column %s skipped (unsupported type %s)",
                 colname, buf);
         PushWarning(g, thd);
         continue;
         } // endif v
 
-      sprintf(g->Message, "Column %s unsupported type %s", colname, buf);
+      snprintf(g->Message, sizeof(g->Message), "Column %s unsupported type %s", colname, buf);
       myc.Close();
       return NULL;
     } else if (type == TYPE_STRING) {
       if (v == 'X') {
         len = GetConvSize();
-        sprintf(g->Message, "Column %s converted to varchar(%d)",
+        snprintf(g->Message, sizeof(g->Message), "Column %s converted to varchar(%d)",
                 colname, len);
         PushWarning(g, thd);
         v = 'V';
-      } else
+			} else
         len = MY_MIN(len, 4096);
 
     } // endif type
@@ -286,6 +311,9 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
       default: crp->Nulls[i] = v;   break;
       } // endswitch nf
 
+		if (b)																 // enum or set
+  		nf = sscanf(fld, "%s ", buf);				 // get values
+
     crp = crp->Next;                       // Type_Name
     crp->Kdata->SetValue(buf, i);
 
@@ -371,14 +399,26 @@ PQRYRES SrcColumns(PGLOBAL g, const char *host, const char *db,
   int     w;
   MYSQLC  myc;
   PQRYRES qrp = NULL;
+  const char *p;
 
   if (!port)
     port = mysqld_port;
 
-  if (!strnicmp(srcdef, "select ", 7)) {
-    query = (char *)PlugSubAlloc(g, NULL, strlen(srcdef) + 9);
-    strcat(strcpy(query, srcdef), " LIMIT 0");
-  } else
+	if (!strnicmp(srcdef, "select ", 7) || strstr(srcdef, "%s")) {
+		query = (char *)PlugSubAlloc(g, NULL, strlen(srcdef) + 10);
+
+		if ((p= strstr(srcdef, "%s")))
+		{
+			/* Replace %s with 1=1 */
+			sprintf(query, "%.*s1=1%s", (int) (p - srcdef), srcdef, p + 2); // dummy where clause
+		}
+		else 
+		  strcpy(query, srcdef);
+
+		if (!strnicmp(srcdef, "select ", 7))
+		  strcat(query, " LIMIT 0");
+
+	} else
     query = (char *)srcdef;
 
   // Open a MySQL connection for this table
@@ -436,7 +476,7 @@ int MYSQLC::Open(PGLOBAL g, const char *host, const char *db,
                             int pt, const char *csname)
   {
   const char *pipe = NULL;
-  uint        cto = 6000, nrt = 12000;
+  //uint      cto = 10, nrt = 20;
   my_bool     my_true= 1;
 
   m_DB = mysql_init(NULL);
@@ -446,25 +486,25 @@ int MYSQLC::Open(PGLOBAL g, const char *host, const char *db,
     return RC_FX;
     } // endif m_DB
 
-	if (trace)
+	if (trace(1))
 		htrc("MYSQLC Open: m_DB=%.4X size=%d\n", m_DB, (int)sizeof(*m_DB));
 
-	// Removed to do like FEDERATED do
+	// Removed to do like FEDERATED does
 //mysql_options(m_DB, MYSQL_READ_DEFAULT_GROUP, "client-mariadb");
-  mysql_options(m_DB, MYSQL_OPT_USE_REMOTE_CONNECTION, NULL);
-  mysql_options(m_DB, MYSQL_OPT_CONNECT_TIMEOUT, &cto);
-  mysql_options(m_DB, MYSQL_OPT_READ_TIMEOUT, &nrt);
+//mysql_options(m_DB, MYSQL_OPT_USE_REMOTE_CONNECTION, NULL);
+//mysql_options(m_DB, MYSQL_OPT_CONNECT_TIMEOUT, &cto);
+//mysql_options(m_DB, MYSQL_OPT_READ_TIMEOUT, &nrt);
 //mysql_options(m_DB, MYSQL_OPT_WRITE_TIMEOUT, ...);
 
-#if defined(__WIN__)
+#if defined(_WIN32)
   if (!strcmp(host, ".")) {
     mysql_options(m_DB, MYSQL_OPT_NAMED_PIPE, NULL);
     pipe = mysqld_unix_port;
     } // endif host
-#else   // !__WIN__
+#else   // !_WIN32
   if (!strcmp(host, "localhost"))
     pipe = mysqld_unix_port;
-#endif  // !__WIN__
+#endif  // !_WIN32
 
 #if 0
   if (pwd && !strcmp(pwd, "*")) {
@@ -489,12 +529,13 @@ int MYSQLC::Open(PGLOBAL g, const char *host, const char *db,
   mysql_options(m_DB, MYSQL_OPT_USE_THREAD_SPECIFIC_MEMORY,
                   (char*)&my_true);
 
-  if (!mysql_real_connect(m_DB, host, user, pwd, db, pt, pipe, CLIENT_MULTI_RESULTS)) {
+  if (!mysql_real_connect(m_DB, host, user, pwd, db, pt, pipe,
+		CLIENT_MULTI_RESULTS | CLIENT_REMEMBER_OPTIONS)) {
 #if defined(_DEBUG)
-    sprintf(g->Message, "mysql_real_connect failed: (%d) %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_real_connect failed: (%d) %s",
                         mysql_errno(m_DB), mysql_error(m_DB));
 #else   // !_DEBUG
-    sprintf(g->Message, "(%d) %s", mysql_errno(m_DB), mysql_error(m_DB));
+    snprintf(g->Message, sizeof(g->Message), "(%d) %s", mysql_errno(m_DB), mysql_error(m_DB));
 #endif  // !_DEBUG
     mysql_close(m_DB);
     m_DB = NULL;
@@ -575,7 +616,7 @@ int MYSQLC::PrepareSQL(PGLOBAL g, const char *stmt)
 #if defined(ALPHA)
   if (!(m_Stmt = mysql_prepare(m_DB, stmt, strlen(stmt)))) {
 
-    sprintf(g->Message, "mysql_prepare failed: %s [%s]",
+    snprintf(g->Message, sizeof(g->Message), "mysql_prepare failed: %s [%s]",
                          mysql_error(m_DB), stmt);
     return -1;
     } // endif m_Stmt
@@ -589,7 +630,7 @@ int MYSQLC::PrepareSQL(PGLOBAL g, const char *stmt)
     } // endif m_Stmt
 
   if (mysql_stmt_prepare(m_Stmt, stmt, strlen(stmt))) {
-    sprintf(g->Message, "mysql_stmt_prepare() failed: (%d) %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_stmt_prepare() failed: (%d) %s",
             mysql_stmt_errno(m_Stmt), mysql_stmt_error(m_Stmt));
     return -3;
     } // endif prepare
@@ -612,11 +653,11 @@ int MYSQLC::BindParams(PGLOBAL g, MYSQL_BIND *bind)
 
 #if defined(ALPHA)
   if (mysql_bind_param(m_Stmt, bind)) {
-    sprintf(g->Message, "mysql_bind_param() failed: %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_bind_param() failed: %s",
                         mysql_stmt_error(m_Stmt));
 #else   // !ALPHA
   if (mysql_stmt_bind_param(m_Stmt, bind)) {
-    sprintf(g->Message, "mysql_stmt_bind_param() failed: %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_stmt_bind_param() failed: %s",
                         mysql_stmt_error(m_Stmt));
 #endif  // !ALPHA
     return RC_FX;
@@ -636,13 +677,13 @@ int MYSQLC::ExecStmt(PGLOBAL g)
 
 #if defined(ALPHA)
   if (mysql_execute(m_Stmt)) {
-    sprintf(g->Message, "mysql_execute() failed: %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_execute() failed: %s",
                         mysql_stmt_error(m_Stmt));
     return RC_FX;
     } // endif execute
 #else   // !ALPHA
   if (mysql_stmt_execute(m_Stmt)) {
-    sprintf(g->Message, "mysql_stmt_execute() failed: %s",
+    snprintf(g->Message, sizeof(g->Message), "mysql_stmt_execute() failed: %s",
                         mysql_stmt_error(m_Stmt));
     return RC_FX;
     } // endif execute
@@ -650,7 +691,7 @@ int MYSQLC::ExecStmt(PGLOBAL g)
 
   // Check the total number of affected rows
   if (mysql_stmt_affected_rows(m_Stmt) != 1) {
-    sprintf(g->Message, "Invalid affected rows by MySQL");
+    snprintf(g->Message, sizeof(g->Message), "Invalid affected rows by MySQL");
     return RC_FX;
     } // endif affected_rows
 
@@ -707,7 +748,7 @@ int MYSQLC::ExecSQL(PGLOBAL g, const char *query, int *w)
       m_Fields = mysql_num_fields(m_Res);
       m_Rows = (!m_Use) ? (int)mysql_num_rows(m_Res) : 0;
 
-			if (trace)
+			if (trace(1))
 				htrc("ExecSQL: m_Res=%.4X size=%d m_Fields=%d m_Rows=%d\n",
 				               m_Res, sizeof(*m_Res), m_Fields, m_Rows);
 
@@ -716,7 +757,7 @@ int MYSQLC::ExecSQL(PGLOBAL g, const char *query, int *w)
   } else {
 //  m_Rows = (int)mysql_affected_rows(m_DB);
     m_Rows = (int)m_DB->affected_rows;
-    sprintf(g->Message, "Affected rows: %d\n", m_Rows);
+    snprintf(g->Message, sizeof(g->Message), "Affected rows: %d\n", m_Rows);
     rc = RC_NF;
   } // endif field count
 
@@ -841,7 +882,8 @@ MYSQL_FIELD *MYSQLC::GetNextField(void)
 /***********************************************************************/
 PQRYRES MYSQLC::GetResult(PGLOBAL g, bool pdb)
   {
-  char        *fmt, v;
+	PCSZ         fmt;
+  char        *name, v= 0;
   int          n;
   bool         uns;
   PCOLRES     *pcrp, crp;
@@ -850,7 +892,7 @@ PQRYRES MYSQLC::GetResult(PGLOBAL g, bool pdb)
   MYSQL_ROW    row;
 
   if (!m_Res || !m_Fields) {
-    sprintf(g->Message, "%s result", (m_Res) ? "Void" : "No");
+    snprintf(g->Message, sizeof(g->Message), "%s result", (m_Res) ? "Void" : "No");
     return NULL;
     } // endif m_Res
 
@@ -879,11 +921,12 @@ PQRYRES MYSQLC::GetResult(PGLOBAL g, bool pdb)
     memset(crp, 0, sizeof(COLRES));
     crp->Ncol = ++qrp->Nbcol;
 
-    crp->Name = (char*)PlugSubAlloc(g, NULL, fld->name_length + 1);
-    strcpy(crp->Name, fld->name);
+    name = (char*)PlugSubAlloc(g, NULL, fld->name_length + 1);
+    strcpy(name, fld->name);
+		crp->Name = name;
 
     if ((crp->Type = MYSQLtoPLG(fld->type, &v)) == TYPE_ERROR) {
-      sprintf(g->Message, "Type %d not supported for column %s",
+      snprintf(g->Message, sizeof(g->Message), "Type %d not supported for column %s",
                           fld->type, crp->Name);
       return NULL;
     } else if (crp->Type == TYPE_DATE && !pdb)
@@ -894,13 +937,14 @@ PQRYRES MYSQLC::GetResult(PGLOBAL g, bool pdb)
 
     crp->Prec = (crp->Type == TYPE_DOUBLE || crp->Type == TYPE_DECIM)
               ? fld->decimals : 0;
-    crp->Length = MY_MAX(fld->length, fld->max_length);
-    crp->Clen = GetTypeSize(crp->Type, crp->Length);
+    CHARSET_INFO *cs= get_charset(fld->charsetnr, MYF(0));
+    crp->Clen = GetTypeSize(crp->Type, fld->length);
+    crp->Length = fld->length / (cs ? cs->mbmaxlen : 1);
     uns = (fld->flags & (UNSIGNED_FLAG | ZEROFILL_FLAG)) ? true : false;
 
     if (!(crp->Kdata = AllocValBlock(g, NULL, crp->Type, m_Rows,
                                      crp->Clen, 0, FALSE, TRUE, uns))) {
-      sprintf(g->Message, MSG(INV_RESULT_TYPE),
+      snprintf(g->Message, sizeof(g->Message), MSG(INV_RESULT_TYPE),
                           GetFormatType(crp->Type));
       return NULL;
     } else if (crp->Type == TYPE_DATE) {
@@ -929,7 +973,7 @@ PQRYRES MYSQLC::GetResult(PGLOBAL g, bool pdb)
   /*********************************************************************/
   for (n = 0; n < m_Rows; n++) {
     if (!(m_Row = mysql_fetch_row(m_Res))) {
-      sprintf(g->Message, "Missing row %d from result", n + 1);
+      snprintf(g->Message, sizeof(g->Message), "Missing row %d from result", n + 1);
       return NULL;
       } // endif m_Row
 
@@ -1007,7 +1051,7 @@ int MYSQLC::ExecSQLcmd(PGLOBAL g, const char *query, int *w)
 //if (mysql_query(m_DB, query) != 0) {
   if (mysql_real_query(m_DB, query, strlen(query))) {
     m_Afrw = (int)mysql_errno(m_DB);
-    sprintf(g->Message, "Remote: %s", mysql_error(m_DB));
+    snprintf(g->Message, sizeof(g->Message), "Remote: %s", mysql_error(m_DB));
     rc = RC_FX;
 //} else if (!(m_Fields = mysql_field_count(m_DB))) {
   } else if (!(m_Fields = (int)m_DB->field_count)) {
@@ -1028,7 +1072,7 @@ void MYSQLC::Close(void)
   {
   FreeResult();
 
-	if (trace)
+	if (trace(1))
 		htrc("MYSQLC Close: m_DB=%.4X\n", m_DB);
 
 	mysql_close(m_DB);
