@@ -30,6 +30,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define MYSQL_SERVER 1
 #include <my_global.h>
 #include "sql_priv.h"
+#include <mysqld_error.h>
 
 #include "ha_federatedx.h"
 
@@ -63,19 +64,18 @@ struct mysql_position
 class federatedx_io_mysql :public federatedx_io
 {
   MYSQL mysql; /* MySQL connection */
-  MYSQL_ROWS *current;
   DYNAMIC_ARRAY savepoints;
   bool requested_autocommit;
   bool actual_autocommit;
 
-  int actual_query(const char *buffer, uint length);
+  int actual_query(const char *buffer, size_t length);
   bool test_all_restrict() const;
 public:
   federatedx_io_mysql(FEDERATEDX_SERVER *);
   ~federatedx_io_mysql();
 
   int simple_query(const char *fmt, ...);
-  int query(const char *buffer, uint length);
+  int query(const char *buffer, size_t length);
   virtual FEDERATEDX_IO_RESULT *store_result();
 
   virtual size_t max_query_size() const;
@@ -107,7 +107,8 @@ public:
   virtual void free_result(FEDERATEDX_IO_RESULT *io_result);
   virtual unsigned int get_num_fields(FEDERATEDX_IO_RESULT *io_result);
   virtual my_ulonglong get_num_rows(FEDERATEDX_IO_RESULT *io_result);
-  virtual FEDERATEDX_IO_ROW *fetch_row(FEDERATEDX_IO_RESULT *io_result);
+  virtual FEDERATEDX_IO_ROW *fetch_row(FEDERATEDX_IO_RESULT *io_result,
+                                       FEDERATEDX_IO_ROWS **current= NULL);
   virtual ulong *fetch_lengths(FEDERATEDX_IO_RESULT *io_result);
   virtual const char *get_column_data(FEDERATEDX_IO_ROW *row,
                                       unsigned int column);
@@ -116,9 +117,10 @@ public:
 
   virtual size_t get_ref_length() const;
   virtual void mark_position(FEDERATEDX_IO_RESULT *io_result,
-                             void *ref);
+                             void *ref, FEDERATEDX_IO_ROWS *current);
   virtual int seek_position(FEDERATEDX_IO_RESULT **io_result,
                             const void *ref);
+  virtual void set_thd(void *thd);
 };
 
 
@@ -271,7 +273,7 @@ ulong federatedx_io_mysql::savepoint_release(ulong sp)
   if (last)
   {
     char buffer[STRING_BUFFER_USUAL_SIZE];
-  int length= my_snprintf(buffer, sizeof(buffer),
+    size_t length= my_snprintf(buffer, sizeof(buffer),
               "RELEASE SAVEPOINT save%lu", last->level);
     actual_query(buffer, length);
   }
@@ -306,7 +308,7 @@ ulong federatedx_io_mysql::savepoint_rollback(ulong sp)
   if (savept && !(savept->flags & SAVEPOINT_RESTRICT))
   {
     char buffer[STRING_BUFFER_USUAL_SIZE];
-  int length= my_snprintf(buffer, sizeof(buffer),
+    size_t length= my_snprintf(buffer, sizeof(buffer),
               "ROLLBACK TO SAVEPOINT save%lu", savept->level);
     actual_query(buffer, length);
   }
@@ -339,7 +341,8 @@ void federatedx_io_mysql::savepoint_restrict(ulong sp)
 int federatedx_io_mysql::simple_query(const char *fmt, ...)
 {
   char buffer[STRING_BUFFER_USUAL_SIZE];
-  int length, error;
+  size_t length;
+  int error;
   va_list arg;
   DBUG_ENTER("federatedx_io_mysql::simple_query");
 
@@ -375,7 +378,7 @@ bool federatedx_io_mysql::test_all_restrict() const
 }
 
 
-int federatedx_io_mysql::query(const char *buffer, uint length)
+int federatedx_io_mysql::query(const char *buffer, size_t length)
 {
   int error;
   bool wants_autocommit= requested_autocommit | is_readonly();
@@ -400,7 +403,7 @@ int federatedx_io_mysql::query(const char *buffer, uint length)
     if (!(savept->flags & SAVEPOINT_RESTRICT))
   {
       char buf[STRING_BUFFER_USUAL_SIZE];
-    int len= my_snprintf(buf, sizeof(buf),
+      size_t len= my_snprintf(buf, sizeof(buf),
                   "SAVEPOINT save%lu", savept->level);
       if ((error= actual_query(buf, len)))
     DBUG_RETURN(error);                         
@@ -417,7 +420,7 @@ int federatedx_io_mysql::query(const char *buffer, uint length)
 }
 
 
-int federatedx_io_mysql::actual_query(const char *buffer, uint length)
+int federatedx_io_mysql::actual_query(const char *buffer, size_t length)
 {
   int error;
   DBUG_ENTER("federatedx_io_mysql::actual_query");
@@ -450,7 +453,8 @@ int federatedx_io_mysql::actual_query(const char *buffer, uint length)
     mysql.reconnect= 1;
   }
 
-  error= mysql_real_query(&mysql, buffer, length);
+  if (!(error= mysql_real_query(&mysql, STRING_WITH_LEN("set time_zone='+00:00'"))))
+    error= mysql_real_query(&mysql, buffer, (ulong)length);
   
   DBUG_RETURN(error);
 }
@@ -513,10 +517,12 @@ my_ulonglong federatedx_io_mysql::get_num_rows(FEDERATEDX_IO_RESULT *io_result)
 }
 
 
-FEDERATEDX_IO_ROW *federatedx_io_mysql::fetch_row(FEDERATEDX_IO_RESULT *io_result)
+FEDERATEDX_IO_ROW *federatedx_io_mysql::fetch_row(FEDERATEDX_IO_RESULT *io_result,
+                                                  FEDERATEDX_IO_ROWS **current)
 {
   MYSQL_RES *result= (MYSQL_RES*)io_result;
-  current= result->data_cursor;
+  if (current)
+    *current= (FEDERATEDX_IO_ROWS *) result->data_cursor;
   return (FEDERATEDX_IO_ROW *) mysql_fetch_row(result);
 }
 
@@ -624,11 +630,11 @@ size_t federatedx_io_mysql::get_ref_length() const
 
 
 void federatedx_io_mysql::mark_position(FEDERATEDX_IO_RESULT *io_result,
-                                        void *ref)
+                                        void *ref, FEDERATEDX_IO_ROWS *current)
 {
   mysql_position& pos= *reinterpret_cast<mysql_position*>(ref);
   pos.result= (MYSQL_RES *) io_result;
-  pos.offset= current;
+  pos.offset= (MYSQL_ROW_OFFSET) current;
 }
 
 int federatedx_io_mysql::seek_position(FEDERATEDX_IO_RESULT **io_result,
@@ -646,3 +652,7 @@ int federatedx_io_mysql::seek_position(FEDERATEDX_IO_RESULT **io_result,
   return 0;
 }
 
+void federatedx_io_mysql::set_thd(void *thd)
+{
+  mysql.net.thd= thd;
+}
