@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2015 Kentoku Shiba
+/* Copyright (C) 2008-2018 Kentoku Shiba
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -11,16 +11,12 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 #ifdef USE_PRAGMA_INTERFACE
 #pragma interface
 #endif
 
-#if (defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000)
-#define SPIDER_HANDLER_START_BULK_INSERT_HAS_FLAGS
-#endif
-
 #define SPIDER_CONNECT_INFO_MAX_LEN 64
 #define SPIDER_CONNECT_INFO_PATH_MAX_LEN FN_REFLEN
 #define SPIDER_LONGLONG_LEN 20
@@ -133,6 +129,14 @@ public:
   bool               da_status;
   bool               use_spatial_index;
 
+#ifdef SPIDER_HAS_GROUP_BY_HANDLER
+  uint                  idx_for_direct_join;
+  bool                  use_fields;
+  spider_fields         *fields;
+  SPIDER_LINK_IDX_CHAIN *link_idx_chain;
+  SPIDER_LINK_IDX_CHAIN *result_link_idx_chain;
+#endif
+
   /* for mrr */
   bool               mrr_with_cnt;
   uint               multi_range_cnt;
@@ -164,7 +168,6 @@ public:
   bool               insert_with_update;
   bool               low_priority;
   bool               high_priority;
-  bool               insert_delayed;
   bool               use_pre_call;
   bool               use_pre_records;
   bool               pre_bitmap_checked;
@@ -251,6 +254,11 @@ public:
   /* for dbton */
   spider_db_handler  **dbton_handler;
 
+  /* for direct limit offset */
+  longlong direct_select_offset;
+  longlong direct_current_offset;
+  longlong direct_select_limit;
+
   ha_spider();
   ha_spider(
     handlerton *hton,
@@ -564,19 +572,51 @@ public:
 #endif
   bool start_bulk_update();
   int exec_bulk_update(
-    uint *dup_key_found
+    ha_rows *dup_key_found
   );
-  void end_bulk_update();
+  int end_bulk_update();
+#ifdef SPIDER_UPDATE_ROW_HAS_CONST_NEW_DATA
+  int bulk_update_row(
+    const uchar *old_data,
+    const uchar *new_data,
+    ha_rows *dup_key_found
+  );
+  int update_row(
+    const uchar *old_data,
+    const uchar *new_data
+  );
+#else
   int bulk_update_row(
     const uchar *old_data,
     uchar *new_data,
-    uint *dup_key_found
+    ha_rows *dup_key_found
   );
   int update_row(
     const uchar *old_data,
     uchar *new_data
   );
+#endif
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+#ifdef SPIDER_MDEV_16246
+  inline int direct_update_rows_init(
+    List<Item> *update_fields
+  ) {
+    return direct_update_rows_init(update_fields, 2, NULL, 0, FALSE, NULL);
+  }
+  int direct_update_rows_init(
+    List<Item> *update_fields,
+    uint mode,
+    KEY_MULTI_RANGE *ranges,
+    uint range_count,
+    bool sorted,
+    uchar *new_data
+  );
+#else
+  inline int direct_update_rows_init()
+  {
+    return direct_update_rows_init(2, NULL, 0, FALSE, NULL);
+  }
   int direct_update_rows_init(
     uint mode,
     KEY_MULTI_RANGE *ranges,
@@ -584,7 +624,37 @@ public:
     bool sorted,
     uchar *new_data
   );
+#endif
+#else
+#ifdef SPIDER_MDEV_16246
+  int direct_update_rows_init(
+    List<Item> *update_fields
+  );
+#else
+  int direct_update_rows_init();
+#endif
+#endif
 #ifdef HA_CAN_BULK_ACCESS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+#ifdef SPIDER_MDEV_16246
+  inline int pre_direct_update_rows_init(
+    List<Item> *update_fields
+  ) {
+    return pre_direct_update_rows_init(update_fields, 2, NULL, 0, FALSE, NULL);
+  }
+  int pre_direct_update_rows_init(
+    List<Item> *update_fields,
+    uint mode,
+    KEY_MULTI_RANGE *ranges,
+    uint range_count,
+    bool sorted,
+    uchar *new_data
+  );
+#else
+  inline int pre_direct_update_rows_init()
+  {
+    return pre_direct_update_rows_init(2, NULL, 0, FALSE, NULL);
+  }
   int pre_direct_update_rows_init(
     uint mode,
     KEY_MULTI_RANGE *ranges,
@@ -593,21 +663,51 @@ public:
     uchar *new_data
   );
 #endif
+#else
+#ifdef SPIDER_MDEV_16246
+  int pre_direct_update_rows_init(
+    List<Item> *update_fields
+  );
+#else
+  int pre_direct_update_rows_init();
+#endif
+#endif
+#endif
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int direct_update_rows(ha_rows *update_rows)
+  {
+    return direct_update_rows(NULL, 0, FALSE, NULL, update_rows);
+  }
   int direct_update_rows(
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted,
     uchar *new_data,
-    uint *update_rows
+    ha_rows *update_rows
   );
+#else
+  int direct_update_rows(
+    ha_rows *update_rows
+  );
+#endif
 #ifdef HA_CAN_BULK_ACCESS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int pre_direct_update_rows()
+  {
+    ha_rows update_rows;
+
+    return pre_direct_update_rows(NULL, 0, FALSE, NULL, &update_rows);
+  }
   int pre_direct_update_rows(
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted,
     uchar *new_data,
-    uint *update_rows
+    ha_rows *update_rows
   );
+#else
+  int pre_direct_update_rows();
+#endif
 #endif
 #endif
   bool start_bulk_delete();
@@ -616,33 +716,69 @@ public:
     const uchar *buf
   );
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int direct_delete_rows_init()
+  {
+    return direct_delete_rows_init(2, NULL, 0, FALSE);
+  }
   int direct_delete_rows_init(
     uint mode,
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted
   );
+#else
+  int direct_delete_rows_init();
+#endif
 #ifdef HA_CAN_BULK_ACCESS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int pre_direct_delete_rows_init()
+  {
+    return pre_direct_delete_rows_init(2, NULL, 0, FALSE);
+  }
   int pre_direct_delete_rows_init(
     uint mode,
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted
   );
+#else
+  int pre_direct_delete_rows_init();
 #endif
+#endif
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int direct_delete_rows(ha_rows *delete_rows)
+  {
+    return direct_delete_rows(NULL, 0, FALSE, delete_rows);
+  }
   int direct_delete_rows(
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted,
-    uint *delete_rows
+    ha_rows *delete_rows
   );
+#else
+  int direct_delete_rows(
+    ha_rows *delete_rows
+  );
+#endif
 #ifdef HA_CAN_BULK_ACCESS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+  inline int pre_direct_delete_rows()
+  {
+    ha_rows delete_rows;
+
+    return pre_direct_delete_rows(NULL, 0, FALSE, &delete_rows);
+  }
   int pre_direct_delete_rows(
     KEY_MULTI_RANGE *ranges,
     uint range_count,
     bool sorted,
-    uint *delete_rows
+    ha_rows *delete_rows
   );
+#else
+  int pre_direct_delete_rows();
+#endif
 #endif
 #endif
   int delete_all_rows();
@@ -751,6 +887,10 @@ public:
   );
   uint check_partitioned();
   void check_direct_order_limit();
+  void check_distinct_key_query();
+  bool is_sole_projection_field(
+    uint16 field_index
+  );
   int check_ha_range_eof();
   int drop_tmp_tables();
   bool handler_opened(
